package model.fixture;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import model.KiwiModelHandler;
import model.generic.PropertyItem;
import model.lightController.LightControllable;

public class Fixture extends PropertyItem{
    private String name;
	private int address;
	private int numberOfChannel = 5;
	private boolean isActive;
	private DmxValues dmxValues;
	private LightControllable lightController;
	private PropertyChangeListener dmxValueListener;
	
	public Fixture(String name, int adress) {
		super();
		this.name = name;
		this.address = adress;
		this.dmxValues = new DmxValues();
		this.lightController = KiwiModelHandler.getLightController();
		this.dmxValueListener = new PropertyChangeListener() {
            @Override
            public void propertyChange(PropertyChangeEvent event) {
            	sendDmxValue();
            }
		};
		this.dmxValues.addPropertyChangeListener(this.dmxValueListener);
	}

	public String getName() {
		return this.name;
	}

	public void setName(String name) {
		this.propertySupport.firePropertyChange("name", this.name, name);
		this.name = name;
	}

	public int getAddress() {
		return this.address;
	}

	public void setAddress(int address) {
		this.propertySupport.firePropertyChange("address", this.address, address);
		this.address = address;
	}

	public boolean isActive() {
		return this.isActive;
	}
	
	public DmxValues getDmxValues() {
		return this.dmxValues;
	}
	
	public void setDmxValues(DmxValues dmxValues) {
		this.dmxValues.removePropertyChangeListener(dmxValueListener);
		this.dmxValues = dmxValues;
		this.dmxValues.addPropertyChangeListener(dmxValueListener);
		if(this.isActive) {
			this.sendDmxValue();
		}
	}
	
	public void activate() {
		if(!this.isActive) {
			this.isActive = true;
			this.propertySupport.firePropertyChange("isActive", false, true);
			this.sendDmxValue();
		}
		
	}
	
	public void deactivate() {
		if(this.isActive) {
			this.isActive = false;
			this.propertySupport.firePropertyChange("isActive", true, false);
			this.sendDmxValue();
		}
	}

	private void sendDmxValue(){
		if(isActive) {
			this.lightController.setDmxValue(this.address, this.dmxValues.getIntensityRed());
			this.lightController.setDmxValue(this.address + 1, this.dmxValues.getIntensityGreen());
			this.lightController.setDmxValue(this.address + 2, this.dmxValues.getIntensityBlue());
			this.lightController.setDmxValue(this.address + 3, this.dmxValues.getIntensityMain());
			this.lightController.setDmxValue(this.address + 4, this.dmxValues.getStropeRate());

			for(int i = 0; i < this.numberOfChannel; i++) {
				this.lightController.activateDmxChannel(this.address + i);
			}
		} else {
			for(int i = 0; i < this.numberOfChannel; i++) {
				this.lightController.sendDmxValue(this.address + i, (byte)0x00);
				this.lightController.deactivateDmxChannel(this.address + i);
			}
		}
	}
}
