package model.fixture;

import java.util.Map;

import model.generic.PropertyItem;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.HashMap;

public class FixtureRepository extends PropertyItem{
	private Map<String, Fixture> fixtures;
	private int activeFixtures;
	private PropertyChangeListener fixtureListener;

	public FixtureRepository() {
		this.fixtures = new HashMap<>();
		this.activeFixtures = 0;
		this.fixtureListener = new PropertyChangeListener() {
            @Override
            public void propertyChange(PropertyChangeEvent event) {
            	if(event.getPropertyName().equals("isActive")) {
					if((boolean) event.getNewValue()) {
						addNumberOfActiveFixtures();
					} else {
						subNumberOfActiveFixtures();
					}
        		}
            }
		};
	}

	public void addFixture(Fixture fixture) {
		if(this.fixtures.containsKey(fixture.getName())) {
			throw new IllegalArgumentException("Es existiert bereits eine Lampe mit diesem Namen");
		}
		
		this.fixtures.put(fixture.getName(), fixture);
		this.propertySupport.firePropertyChange("fixtures", null, fixture);

		fixture.addPropertyChangeListener(this.fixtureListener);
	}	

	private void addNumberOfActiveFixtures() {
		this.propertySupport.firePropertyChange("activeFixtures", this.activeFixtures, this.activeFixtures+1);
		this.activeFixtures++;
	}

	private void subNumberOfActiveFixtures() {
		this.propertySupport.firePropertyChange("activeFixtures", this.activeFixtures, this.activeFixtures-1);
		this.activeFixtures--;
	}

	public int getActiveFixturesCount() {
		return this.activeFixtures;
	}
	
	public Fixture getFixture(String name) {
		return this.fixtures.get(name);
	}
	
	public void removeFixture(String name) {
		this.fixtures.get(name).deactivate();
		this.fixtures.get(name).removePropertyChangeListener(fixtureListener);
		this.propertySupport.firePropertyChange("fixtures", this.fixtures.get(name), null);
		this.fixtures.remove(name);
	}
	
	public Map<String, Fixture> getFixtureMap() {
		return this.fixtures;
	}
	
	
	
	public void clearRepro() {
		Map.copyOf(this.fixtures).forEach((String name, Fixture fixture) -> {
			this.removeFixture(name);
		});
		this.activeFixtures = 0;
	}
}
