﻿package model.lightController.artNet;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.SocketException;
import java.util.concurrent.atomic.AtomicBoolean;
import com.google.gson.Gson;
import model.lightController.LightControllable;
import model.lightController.artNet.artNetCommands.ActivateGroupCommand;
import model.lightController.artNet.artNetCommands.ActivatePresetCommand;
import model.lightController.artNet.artNetCommands.ClearPresetCommand;
import model.lightController.artNet.artNetCommands.DeactivateGroupCommand;
import model.lightController.artNet.artNetCommands.DeactivatePresetCommand;
import model.lightController.artNet.artNetCommands.SetChannelStateCommand;
import model.lightController.artNet.artNetCommands.SetPresetCommand;
import model.lightController.artNet.artNetNode.ArtNetNode;
import model.lightController.artNet.artNetNode.ArtNetNodeRepository;
import model.lightController.artNet.artNetPakages.ArtCommand;
import model.lightController.artNet.artNetPakages.ArtDataRequest;
import model.lightController.artNet.artNetPakages.ArtDmx;
import model.lightController.artNet.artNetPakages.ArtNetCodeLibrary;
import model.lightController.artNet.artNetPakages.ArtPoll;
import model.lightController.artNet.udpListener.UdpListener;
import model.preset.PresetRepository;


public class ArtNetController implements LightControllable {

    private ArtNetNodeRepository artNetNodeRepository;
    private UdpListener udpListener;
    private PresetRepository presetRepository;
    private Thread udpListenerTherad;
    private byte[] dmxValues;
    private int artNodePort = 6454;
    private final AtomicBoolean runningUdpListener = new AtomicBoolean(false);
    Gson gson;
    
    public ArtNetController(PresetRepository presetRepository) {
    	super();
    	this.presetRepository = presetRepository;
        this.artNetNodeRepository = new ArtNetNodeRepository();
        this.dmxValues = new byte[100];
        this.gson = new Gson();
        this.udpListener = new UdpListener(this, artNodePort, this.presetRepository, this.runningUdpListener);
        
        
        
        
    	//ArtNetNode artNetNode = new ArtNetNode("GlobalNode", "192.168.178.109", artNodePort);
      //ArtNetNode artNetNode = new ArtNetNode("Arduino", "192.168.0.101", artNodePort);
    	ArtNetNode artNetNode = new ArtNetNode("Enttec", "192.168.0.100", artNodePort);
        this.artNetNodeRepository.addNode(artNetNode);
    }
    
    private void sendUdpForEachNode(byte[] udpSendData) {
    	this.artNetNodeRepository.getNodesMap().forEach((String name, ArtNetNode node) -> {
            if(node.getNet() == 0 && node.getSubNet() == 0 && node.getUniverse() == 0 && node.isConnected()) {
                try(DatagramSocket socket = new DatagramSocket()) {
                    InetAddress reciverIP = InetAddress.getByName(node.getIp());
                    DatagramPacket outgoingDatagram = new DatagramPacket(udpSendData, udpSendData.length, reciverIP, node.getPort());
                    socket.send(outgoingDatagram);
                } catch (SocketException e) {
                    e.printStackTrace(System.err);
                } catch (IOException e) {
                    e.printStackTrace(System.err);
                }
            }
        });
    }
    
    private void sendArtCommand(ArtCommand artCommand) {
        this.sendUdpForEachNode(artCommand.getUdpArray());
    }
    
    private void sendArtPoll() {
    	System.out.println("ArtPoll senden.");
        byte[] udpSendData = (new ArtPoll()).getUdpArray();
        
        this.artNetNodeRepository.getNodesMap().forEach((String name, ArtNetNode node) -> {
            if(node.getNet() == 0 && node.getSubNet() == 0 && node.getUniverse() == 0) {
                try(DatagramSocket socket = new DatagramSocket()) {
                	byte[] broadcast = InetAddress.getLocalHost().getAddress();
                	System.out.println(InetAddress.getLocalHost());
                	broadcast[3] = (byte)255;
                    InetAddress reciverIP = InetAddress.getByAddress(broadcast);
                    DatagramPacket outgoingDatagram = new DatagramPacket(udpSendData, udpSendData.length, reciverIP, node.getPort());
                    socket.send(outgoingDatagram);
                } catch (SocketException e) {
                    e.printStackTrace(System.err);
                } catch (IOException e) {
                    e.printStackTrace(System.err);
                }
            }
        });
    }
    
    private void sendDataRequest() {
    	System.out.println("ArtDataRequest senden.");
        ArtDataRequest artDataRequest = new ArtDataRequest(ArtNetCodeLibrary.RequestCodes.DR_PRESET_STATUS.requestCode);
        this.sendUdpForEachNode(artDataRequest.getUdpArray());
    }
    
    @Override
    public void setDmxValue(int channel, byte value)
    {
        //System.out.println(String.format("Channel %d wurde auf %d gesetzt", channel, Byte.toUnsignedInt(value)));
        this.dmxValues[channel-1] = value;
    }

    @Override
    public void sendDmxValue() {
        //System.out.println("DMX-Daten werden gesendet.");
        byte[] udpSendData = (new ArtDmx(this.dmxValues)).getUdpArray();
        
        this.sendUdpForEachNode(udpSendData);
    }

    @Override
    public void sendDmxValue(int channel, byte value) {
        this.dmxValues[channel-1] = value;
        sendDmxValue();
    }

    @Override
    public void activateDmxChannel(int channel) {
        //System.out.println(String.format("Channel %d wurde aktiviert", channel));
        SetChannelStateCommand setChannelStateCommand = new SetChannelStateCommand(channel, true);
        String command = ArtNetCodeLibrary.ArtCommands.ART_COM_SC.command + "=" + this.gson.toJson(setChannelStateCommand);
        sendArtCommand(new ArtCommand(command));
    }

    @Override
    public void deactivateDmxChannel(int channel) {
        //System.out.println(String.format("Channel %d wurde deaktiviert", channel));
        SetChannelStateCommand setChannelStateCommand = new SetChannelStateCommand(channel, false);
        String command = ArtNetCodeLibrary.ArtCommands.ART_COM_SC.command + "=" + this.gson.toJson(setChannelStateCommand);
        sendArtCommand(new ArtCommand(command));
    }

    @Override
    public void activateGroup(byte groupId) {
        //System.out.println(String.format("Gruppe %d wurde aktiviert", Byte.toUnsignedInt(groupId)));
        ActivateGroupCommand activateGroupCommand = new ActivateGroupCommand(groupId);
        String command = ArtNetCodeLibrary.ArtCommands.ART_COM_AG.command + "=" + this.gson.toJson(activateGroupCommand);
        sendArtCommand(new ArtCommand(command));
    }

    @Override
    public void deactivateGroup(byte groupId) {
        //System.out.println(String.format("Gruppe %d wurde deaktiviert", Byte.toUnsignedInt(groupId)));
        DeactivateGroupCommand deactivateGroupCommand = new DeactivateGroupCommand(groupId);
        String command = ArtNetCodeLibrary.ArtCommands.ART_COM_DG.command + "=" + this.gson.toJson(deactivateGroupCommand);
        sendArtCommand(new ArtCommand(command));
    }

    @Override
    public void activatePreset(int presetId) {
        System.out.println(String.format("Preset %d wurde aktiviert", presetId));
        ActivatePresetCommand activatePresetCommand = new ActivatePresetCommand(presetId);
        String command = ArtNetCodeLibrary.ArtCommands.ART_COM_AP.command + "=" + this.gson.toJson(activatePresetCommand);
        sendArtCommand(new ArtCommand(command));
    }

    @Override
    public void deactivatePreset(int presetId) {
        System.out.println(String.format("Preset %d wurde deaktiviert", presetId));
        DeactivatePresetCommand deactivatePresetCommand = new DeactivatePresetCommand(presetId);
        String command = ArtNetCodeLibrary.ArtCommands.ART_COM_DP.command + "=" + this.gson.toJson(deactivatePresetCommand);
        sendArtCommand(new ArtCommand(command));
    }

    @Override
    public void clearPreset(int presetId) {
        System.out.println(String.format("Preset %d wird gelöscht", presetId));
        ClearPresetCommand clearPresetCommand = new ClearPresetCommand(presetId);
        String command = ArtNetCodeLibrary.ArtCommands.ART_COM_CP.command + "=" + this.gson.toJson(clearPresetCommand);
        sendArtCommand(new ArtCommand(command));
    }

    @Override
    public void setPreset(int presetId, byte groupId, int channel, byte value) {
        System.out.println(String.format("Preset %d wird gesetzt", presetId));
        SetPresetCommand setPresetCommand = new SetPresetCommand(presetId, Byte.toUnsignedInt(groupId), channel, Byte.toUnsignedInt(value));
        String command = ArtNetCodeLibrary.ArtCommands.ART_COM_SP.command + "=" + this.gson.toJson(setPresetCommand);
        sendArtCommand(new ArtCommand(command));
    }
    
    @Override
    public void connectToNodes() {
    	udpListenerTherad = new Thread(this.udpListener);
    	this.runningUdpListener.set(true);
    	udpListenerTherad.start();
    	this.sendArtPoll();
    	try {
			Thread.sleep(500);
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
    	if(!this.artNetNodeRepository.isNodesConnected()) {
    		this.artNetNodeRepository.getNodesMap().forEach((String name, ArtNetNode node) -> {
    			if(!node.isConnected()) {
    				node.setConnectStatus(false);
    			}
    		});
    	}
    	this.sendDataRequest();
    }
    
    public void disconnectNodes() {
    	this.runningUdpListener.set(false);
    }
    
    public ArtNetNodeRepository getNodeRepository() {
    	return this.artNetNodeRepository;
    }
}
