package model.lightController.artNet.artNetNode;

import java.util.Map;

import model.generic.PropertyItem;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.HashMap;

public class ArtNetNodeRepository extends PropertyItem{
	private Map<String, ArtNetNode> nodes;
	private boolean nodesConnected = false;
	private int numberOfConnectedNodes = 0;
	private PropertyChangeListener nodesListener;

	public ArtNetNodeRepository() {
		super();
		nodes = new HashMap<>();
		this.nodesListener = new PropertyChangeListener() {
        	@Override
            public void propertyChange(PropertyChangeEvent event) {
            	if(event.getPropertyName().equals("connected")) {
            		if((boolean) event.getNewValue()) {
            			numberOfConnectedNodes++;
            		} else {
            			if(numberOfConnectedNodes >0) {
            				numberOfConnectedNodes--;
            			}
            		}
            		if(numberOfConnectedNodes == nodes.size()) {
        				setNodesConnected(true);
        			} else if(numberOfConnectedNodes == 0) {
        				setNodesConnected(false);
        			}
        		}
            }
        };
	}
	
	public void setNodesConnected(boolean connected) {
//		if(this.nodesConnected != connected) {
		
		this.propertySupport.firePropertyChange("nodesConnected", null, connected);
		this.nodesConnected = connected;
//		}
	}
	
	public void addNode(ArtNetNode artNetNode) {
		artNetNode.addPropertyChangeListener(this.nodesListener);
		nodes.put(artNetNode.getName(), artNetNode);
		this.propertySupport.firePropertyChange("nodes", null, artNetNode);
	}
	
	public void removeNode(ArtNetNode artNetNode) {
		this.propertySupport.firePropertyChange("nodes", artNetNode,null);
		nodes.remove(artNetNode.getName(), artNetNode);
	}
	
	public boolean isNodesConnected() {
		return this.nodesConnected;
	}
	
	public ArtNetNode getNode(String name) {
		return this.nodes.get(name);
	}
	
	public Map<String, ArtNetNode> getNodesMap() {
		return this.nodes;
	}
}
