package model.lightController.artNet.artNetPakages;

import java.nio.ByteBuffer;

public class ArtCommand {
    private String id = "Art-Net" + (char)0x00;
    private short opCode = ArtNetCodeLibrary.OpCodes.OP_COMMAND.opCode;
    private byte protVerHi = 0x00;
    private byte protVerLo = 0x0e; // Version 14;
    private byte estaManHi = 0x00;
    private byte estaManLo = 0x00;
    private short length = 0x0000;
    private String data;

    public ArtCommand(String data) {
        this.length = (short) data.length();
        this.data = data;
    }

    public ArtCommand(byte[] udpArray) {
        byte[] lengthArray = new byte[2];
        char[] data = new char[udpArray.length - 16];

        // protocoll version
        this.protVerHi = udpArray[10];
        this.protVerLo = udpArray[11];

        // esta manifactur
        this.estaManHi = udpArray[12];
        this.estaManLo = udpArray[13];

        // length
        lengthArray[0] = udpArray[14];
        lengthArray[1] = udpArray[15];
        this.length = ByteBuffer.wrap(lengthArray).getShort(); 

        for(int i=0; i < this.length; i++)
        {
            data[i] = (char)udpArray[16+i];
        }
        this.data = String.copyValueOf(data);
    }

    public byte[] getUdpArray() {
        byte[] udpArray = new byte[16 + this.data.length()];
        byte[] opCodeArray = ByteBuffer.allocate(2).putShort(this.opCode).array();
        byte[] lengthArray = ByteBuffer.allocate(2).putShort(this.length).array();

        // id
        for(int i = 0; i < 8; i++) {
            udpArray[i] = (byte)this.id.charAt(i);
        }

        // op code
        udpArray[8] = opCodeArray[1];
        udpArray[9] = opCodeArray[0];

        // protocoll version
        udpArray[10] = this.protVerHi;
        udpArray[11] = this.protVerLo;

        // esta manifactur
        udpArray[12] = this.estaManHi;
        udpArray[13] = this.estaManLo;

        // length
        udpArray[14] = lengthArray[0];
        udpArray[15] = lengthArray[1]; 

        for(int i=0; i < this.length; i++) {
            udpArray[16+i] = this.data.getBytes()[i];
        }
        
        return udpArray;
    }

	public String getId() {
		return id;
	}

	public short getOpCode() {
		return opCode;
	}

	public byte getProtVerHi() {
		return protVerHi;
	}

	public byte getProtVerLo() {
		return protVerLo;
	}

	public byte getEstaManHi() {
		return estaManHi;
	}

	public byte getEstaManLo() {
		return estaManLo;
	}

	public short getLength() {
		return length;
	}

	public String getData() {
		return data;
	}
}
