package model.lightController.artNet.artNetPakages;

import java.net.InetAddress;
import java.net.NetworkInterface;
import java.net.SocketException;
import java.net.UnknownHostException;
import java.nio.ByteBuffer;

public class ArtPollReply {
    private String id = "Art-Net" + (char)0x00;
    private short opCode = ArtNetCodeLibrary.OpCodes.OP_POLL_REPLY.opCode;
    private byte ipAddress[] = {0x00, 0x00, 0x00, 0x00};
    private short port = 0x1936;
    private byte versInfoH = 0x00;
    private byte versInfoL = 0x01;
    private byte netSwitch = 0x00;
    private byte subSwitch = 0x00;
    private byte oemHi = (byte)0xff;
    private byte oemLo = (byte)0xff;
    private byte ubeaVersion = 0x00;
    private byte status1 = 0x00;
    private byte estaManLo = 0x00;
    private byte estaManHi = 0x00;
    private byte portName[] = {'K', 'I', 'W', 'I', 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
    private byte longName[] = new byte[64];
    private byte nodeReport[] = new byte[64];
    private byte numPortsHi = 0x00;
    private byte numPortsLo = 0x01;
    private byte portTypes[] = {0x00, 0x00, 0x00, 0x00};
    private byte goodInput[] = {0x04, 0x04, 0x04, 0x04};
    private byte goodOutputA [] = {0x00, 0x00, 0x00, 0x00};
    private byte swIn[] = {0x00, 0x00, 0x00, 0x00};
    private byte swOut[] = {0x00, 0x00, 0x00, 0x00};
    private byte acnPriority = 0x00;
    private byte swMacro = 0x00;
    private byte swRemote  = 0x00;
    private byte style = 0x00;
    private byte mac[] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
    private byte bindIp[] = {0x00, 0x00, 0x00, 0x00};
    private byte bindIndex = 0x00;
    private byte status2 = 0x00;
    private byte goodOutputB[] = {0x00, 0x00, 0x00, 0x00};
    private byte status3 = 0x00;
    private byte defaulRespUID[] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
    private byte userHi = 0x00;
    private byte userLo = 0x00;
    private byte refreshRateHi = 0x00;
    private byte refreshRateLo = 0x2C;

    public ArtPollReply() {
        try {
			InetAddress localHost = InetAddress.getLocalHost();
            this.ipAddress = localHost.getAddress();
            this.mac = NetworkInterface.getByInetAddress(localHost).getHardwareAddress();
		} catch (UnknownHostException | SocketException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
        
        byte[] longName = "KIWI (is a) Illumination Weeny Interface".getBytes();
        for(int i = 0; i < longName.length; i++) {
            this.longName[i] = longName[i];
        }
    }

    public ArtPollReply(byte[] udpArray) {
        byte[] portArray = new byte[2];

        if(udpArray.length >= 207) {

            // node ip
            for(int i= 0; i < 4; i++)
            {
                this.ipAddress[i] = udpArray[10+i];
            }

            // port
            portArray[1] = udpArray[14];
            portArray[0] = udpArray[15];
            this.port = ByteBuffer.wrap(portArray).getShort();

            // version info
            this.versInfoH = udpArray[16];
            this.versInfoL = udpArray[17];

            // net address
            this.netSwitch = udpArray[18];

            // sub net address
            this.subSwitch = udpArray[19];

            // oem
            this.oemHi = udpArray[20];
            this.oemLo = udpArray[21];

            // ubea version
        this.ubeaVersion =  udpArray[22];

            // status
            this.status1 = udpArray[23];

            // esta
            this.estaManLo = udpArray[24];
            this.estaManHi = udpArray[25];

            // port name
            for (int i = 0; i < 18; i++)
            {
                this.portName[i] = udpArray[26+i];
            }

            // long name
            for (int i = 0; i < 64; i++)
            {
                this.longName[i] = udpArray[44+i];
            }

            // node report
            for (int i = 0; i < 64; i++)
            {
                this.nodeReport[i] = udpArray[108+i];
            }

            // number of ports
            this.numPortsHi = udpArray[172];
            this.numPortsLo = udpArray[173];

            // port type
            for (int i = 0; i < 4; i++)
            {
                this.portTypes[i] = udpArray[174+i];
            }

            // good inputs
            for (int i = 0; i < 4; i++)
            {
                this.goodInput[i] = udpArray[178+i];
            }

            // good output A
            for (int i = 0; i < 4; i++)
            {
                this.goodOutputA[i] = udpArray[182+i];
            }

            // sw in
            for (int i = 0; i < 4; i++)
            {
                this.swIn[i] = udpArray[186+i];
            }

            // sw out
            for (int i = 0; i < 4; i++)
            {
                this.swOut[i] = udpArray[190+i];
            }

            // prio acn
            this.acnPriority = udpArray[194];

            // sw macro
            this.swMacro = udpArray[195];

            // sw remote
            this.swRemote = udpArray[196];

            // style
            this.style = udpArray[200];

            // mac address
            for (int i = 0; i < 6; i++)
            {
                this.mac[i] = udpArray[201+i];
            }
        }
        if(udpArray.length >= 208) {
            // bind ip
            for (int i = 0; i < 4; i++) {
                this.bindIp[i] = udpArray[207+i];
            }
        }
        if(udpArray.length >= 212) {
            // bind index
            this.bindIndex = udpArray[211];
        }
        if(udpArray.length >= 213) {
            // status 2
            this.status2 = udpArray[212];
        }
        if(udpArray.length >= 214) {
            // good output b
            for (int i = 0; i < 4; i++)
            {
                this.goodOutputB[i] = udpArray[213+i];
            }
        }
        if(udpArray.length >= 218) {
            // status 3
            this.status3 = udpArray[217];
        }
        if(udpArray.length >= 219) {
            // default responder uid
            for (int i = 0; i < 6; i++)
            {
                this.defaulRespUID[i] = udpArray[218+i];
            }
        }
        if(udpArray.length >= 225) {
            // user data
            this.userHi = udpArray[224];
            this.userLo = udpArray[225];
        }
        if(udpArray.length >= 227) {
            // user data
            this.refreshRateHi = udpArray[226];
            this.refreshRateLo = udpArray[227];
        }
    }

    public byte[] getUdpArray() {
        byte[] udpArray = new byte[236];
        byte[] opCodeArray = ByteBuffer.allocate(2).putShort(this.opCode).array();
        byte[] portArray = ByteBuffer.allocate(2).putShort(this.port).array();

        // id
        for(int i = 0; i < 8; i++) {
            udpArray[i] = (byte)this.id.charAt(i);
        }

        // op code
        udpArray[8] = opCodeArray[1];
        udpArray[9] = opCodeArray[0];

        // node ip
        for(int i= 0; i < 4; i++)
        {
            udpArray[10+i] = this.ipAddress[i];
        }

        // port
        udpArray[14] = portArray[1];
        udpArray[15] = portArray[0];

        // version info
        udpArray[16] = this.versInfoH;
        udpArray[17] = this.versInfoL;

        // net address
        udpArray[18] = this.netSwitch;

        // sub net address
        udpArray[19] = this.subSwitch;

        // oem
        udpArray[20] = this.oemHi;
        udpArray[21] = this.oemLo;

        // ubea version
        udpArray[22] = this.ubeaVersion;

        // status
        udpArray[23] = this.status1;

        // esta
        udpArray[24] = this.estaManLo;
        udpArray[25] = this.estaManHi;

        // port name
        for (int i = 0; i < 18; i++)
        {
            udpArray[26+i] = this.portName[i];
        }

        // long name
        for (int i = 0; i < 64; i++)
        {
            udpArray[44+i] = this.longName[i];
        }

        // node report
        for (int i = 0; i < 64; i++)
        {
            udpArray[108+i] = this.nodeReport[i];
        }

        // number of ports
        udpArray[172] = this.numPortsHi;
        udpArray[173] = this.numPortsLo;

        // port type
        for (int i = 0; i < 4; i++)
        {
            udpArray[174+i] = this.portTypes[i];
        }

        // good inputs
        for (int i = 0; i < 4; i++)
        {
            udpArray[178+i] = this.goodInput[i];
        }

        // good output A
        for (int i = 0; i < 4; i++)
        {
            udpArray[182+i] = this.goodOutputA[i];
        }

        // sw in
        for (int i = 0; i < 4; i++)
        {
            udpArray[186+i] = this.swIn[i];
        }

        // sw out
        for (int i = 0; i < 4; i++)
        {
            udpArray[190+i] = this.swOut[i];
        }

        // prio acn
        udpArray[194] = this.acnPriority;

        // sw macro
        udpArray[195] = this.swMacro;

        // sw remote
        udpArray[196] = this.swRemote;

        // spare
        for (int i = 0; i < 3; i++)
        {
            udpArray[197+i] = 0x00;
        }

        // style
        udpArray[200] = this.style;

        // mac address
        for (int i = 0; i < 6; i++)
        {
            udpArray[201+i] = this.mac[i];
        }

        // bind ip
        for (int i = 0; i < 4; i++)
        {
            udpArray[207+i] = this.bindIp[i];
        }

        // bind index
        udpArray[211] = this.bindIndex;

        // status 2
        udpArray[212] = this.status2;

        // good output b
        for (int i = 0; i < 4; i++)
        {
            udpArray[213+i] = this.goodOutputB[i];
        }

        // status 3
        udpArray[217] = this.status3;

        // default responder uid
        for (int i = 0; i < 6; i++)
        {
            udpArray[218+i] = this.defaulRespUID[i];
        }

        // user data
        udpArray[224] = this.userHi;
        udpArray[225] = this.userLo;

        // user data
        udpArray[226] = this.refreshRateHi;
        udpArray[227] = this.refreshRateLo;

        // filler
        for (int i = 0; i < 8; i++)
        {
            udpArray[228+i] = 0x00;
        }

        return udpArray;
    }

	public String getId() {
		return id;
	}

	public short getOpCode() {
		return opCode;
	}

	public byte[] getIpAddress() {
		return ipAddress;
	}

	public short getPort() {
		return port;
	}

	public byte getVersInfoH() {
		return versInfoH;
	}

	public byte getVersInfoL() {
		return versInfoL;
	}

	public byte getNetSwitch() {
		return netSwitch;
	}

	public byte getSubSwitch() {
		return subSwitch;
	}

	public byte getOemHi() {
		return oemHi;
	}

	public byte getOemLo() {
		return oemLo;
	}

	public byte getUbeaVersion() {
		return ubeaVersion;
	}

	public byte getStatus1() {
		return status1;
	}

	public byte getEstaManLo() {
		return estaManLo;
	}

	public byte getEstaManHi() {
		return estaManHi;
	}

	public byte[] getPortName() {
		return portName;
	}

	public byte[] getLongName() {
		return longName;
	}

	public byte[] getNodeReport() {
		return nodeReport;
	}

	public byte getNumPortsHi() {
		return numPortsHi;
	}

	public byte getNumPortsLo() {
		return numPortsLo;
	}

	public byte[] getPortTypes() {
		return portTypes;
	}

	public byte[] getGoodInput() {
		return goodInput;
	}

	public byte[] getGoodOutputA() {
		return goodOutputA;
	}

	public byte[] getSwIn() {
		return swIn;
	}

	public byte[] getSwOut() {
		return swOut;
	}

	public byte getAcnPriority() {
		return acnPriority;
	}

	public byte getSwMacro() {
		return swMacro;
	}

	public byte getSwRemote() {
		return swRemote;
	}

	public byte getStyle() {
		return style;
	}

	public byte[] getMac() {
		return mac;
	}

	public byte[] getBindIp() {
		return bindIp;
	}

	public byte getBindIndex() {
		return bindIndex;
	}

	public byte getStatus2() {
		return status2;
	}

	public byte[] getGoodOutputB() {
		return goodOutputB;
	}

	public byte getStatus3() {
		return status3;
	}

	public byte[] getDefaulRespUID() {
		return defaulRespUID;
	}

	public byte getUserHi() {
		return userHi;
	}

	public byte getUserLo() {
		return userLo;
	}

	public byte getRefreshRateHi() {
		return refreshRateHi;
	}

	public byte getRefreshRateLo() {
		return refreshRateLo;
	}
}
