package model.preset;

import model.KiwiModelHandler;
import model.fixture.DmxValues;
import model.fixture.Fixture;
import model.generic.PropertyItem;
import model.group.Group;
import model.lightController.LightControllable;
import model.lightingScene.LightingScene;
import model.lightingScene.LightingScene.LightingSceneType;
import model.lightingScene.composition.Composition;
import model.lightingScene.lightingMood.LightingMood;

public class Preset extends PropertyItem{
	private static int numberOfPresets = 0;
	private int id;
	private Group group;
	private LightingScene lightingScene;
	private State state = State.DEAKTIV;
	private LightControllable lightController = KiwiModelHandler.getLightController();
	
	public static enum State {
		AKTIV,
		PARTAKTIV,
		DEAKTIV
	}
	
	public Preset() {
		this.id = Preset.numberOfPresets;
		Preset.numberOfPresets++;
	}

	public Preset(Group group, LightingScene lightingScene) {
		super();
		this.id = Preset.numberOfPresets;
		this.group = group;
		this.lightingScene = lightingScene;
		Preset.numberOfPresets++;
	}
	
	public Group getGroup() {
		return group;
	}

	public void setGroup(Group group) {
		this.group = group;
	}

	public LightingScene getLightingScene() {
		return lightingScene;
	}

	public void setLightingScene(LightingScene lightingScene) {
		this.lightingScene = lightingScene;
	}

	public int getId() {
		return id;
	}

	public State getState() {
		return state;
	}

	private void setState(State state) {
		this.propertySupport.firePropertyChange("state", this.state, state);
		this.state = state;
	}
	
	public void activate() {
		this.setState(State.AKTIV);
		this.group.setCurrentState(Group.State.AKTIV);
		this.lightingScene.activate();
	}
	
	public void deactivate() {
		this.setState(State.DEAKTIV);
		this.group.setCurrentState(Group.State.DEAKTIV);
		this.lightingScene.deactivate();
	}
	
	public void updatePreset() {
		if(lightingScene != null) {
			this.lightController.clearPreset(this.id);
			if(lightingScene.getLightingSceneType() == LightingSceneType.LIGHTING_MOOD) {
				group.getFixtureRepository().getFixtureMap().forEach((String name, Fixture fixture) -> {
					this.setDmxValueForPreset(fixture.getAddress(), ((LightingMood)lightingScene).getDmxValues());
				});
			} else if (lightingScene.getLightingSceneType() == LightingSceneType.COMPOSITION) {
				group.getFixtureRepository().getFixtureMap().forEach((String name, Fixture fixture) -> {
					this.setDmxValueForPreset(fixture.getAddress(), ((Composition)lightingScene).getDmxValuesForFixture(name));
				});
			}
		}
	}
	
	private void setDmxValueForPreset(int address, DmxValues dmxValues) {
		System.out.println(String.format("Preset %d, hat einen Blauwert von: %d.", this.group.getId(), Byte.toUnsignedInt(dmxValues.getIntensityBlue())));
		lightController.setPreset(this.id, this.group.getId(), address, dmxValues.getIntensityRed());
		lightController.setPreset(this.id, this.group.getId(), address+1, dmxValues.getIntensityGreen());
		lightController.setPreset(this.id, this.group.getId(), address+2, dmxValues.getIntensityBlue());
		lightController.setPreset(this.id, this.group.getId(), address+3, dmxValues.getIntensityMain());
		lightController.setPreset(this.id, this.group.getId(), address+4, dmxValues.getStropeRate());
	}
	
	@Override
	protected void finalize() {
		Preset.numberOfPresets--;
	}
	
	
}
