package view.composition;

import java.io.IOException;
import java.util.Optional;

import javafx.beans.property.BooleanProperty;
import javafx.beans.property.SimpleBooleanProperty;
import javafx.fxml.FXMLLoader;
import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.scene.control.Alert;
import javafx.scene.control.Button;
import javafx.scene.control.ButtonType;
import javafx.scene.control.Label;
import javafx.scene.control.Alert.AlertType;
import javafx.scene.control.ButtonBar.ButtonData;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import javafx.scene.layout.AnchorPane;
import javafx.scene.layout.FlowPane;
import javafx.scene.layout.HBox;
import javafx.scene.layout.Priority;
import javafx.scene.layout.VBox;
import javafx.stage.Stage;
import view.composition.editDeletePanel.EditDeleteCompositionController;
import view.elements.ThreeStageSwitchButton;
import viewModel.KiwiViewModelHandler;
import viewModel.composition.CompositionVM;
import viewModel.group.GroupVM;
import viewModel.group.GroupVM.State;

public class CompositionListItem extends AnchorPane{
	
	private CompositionVM compositionVM;
	private String compositionName;
	private BooleanProperty selected;
	private GroupVM groupVM;
	
	public CompositionListItem(CompositionVM compositionVM, GroupVM groupVM, KiwiViewModelHandler viewModelHandler) {
		this.compositionVM = compositionVM;
		this.groupVM = groupVM;
		this.compositionName = compositionVM.getCompositionName();
		selected = new SimpleBooleanProperty(false);
		
		setMaxHeight(80.0);
		setMinHeight(80.0);
		setPrefHeight(80.0);
		setMaxWidth(200.0);
		setMinWidth(200.0);
		setPrefWidth(200.0);
		FlowPane.setMargin(this, new Insets(5.0, 5.0, 5.0, 5.0));
		
		VBox vBox = new VBox();
		vBox.setAlignment(Pos.CENTER_LEFT);
		vBox.setLayoutX(7.0);
		vBox.setLayoutY(2.0);
		vBox.setMaxWidth(100.0);
		AnchorPane.setBottomAnchor(vBox, 1.0);
		AnchorPane.setLeftAnchor(vBox, 5.0);
		AnchorPane.setRightAnchor(vBox, 5.0);
		AnchorPane.setTopAnchor(vBox, 1.0);
		
		Label lableCompositionName = new Label(compositionName);
		lableCompositionName.setAlignment(Pos.TOP_LEFT);
		lableCompositionName.setMaxHeight(34.0);
		lableCompositionName.setWrapText(true);
		lableCompositionName.textProperty().bind(this.compositionVM.compositionNameProperty());
		VBox.setVgrow(lableCompositionName, Priority.ALWAYS);
		VBox.setMargin(lableCompositionName, new Insets(5.0, 0.0, 0.0, 0.0));
		
		AnchorPane buttonArea = new AnchorPane();
		
		VBox vBoxSwitchButton = new VBox();
		vBoxSwitchButton.setAlignment(Pos.CENTER_LEFT);
		vBoxSwitchButton.setLayoutX(-1.0);
		vBoxSwitchButton.setLayoutY(-1.0);
		AnchorPane.setBottomAnchor(vBoxSwitchButton, 0.0);
		AnchorPane.setLeftAnchor(vBoxSwitchButton, 5.0);
		AnchorPane.setTopAnchor(vBoxSwitchButton, 0.0);
		
		ThreeStageSwitchButton switchButton = new ThreeStageSwitchButton();
		switchButton.setMaxHeight(20.0);
		switchButton.setMinHeight(20.0);
		switchButton.setPrefHeight(20.0);
		switchButton.setMaxWidth(45.0);
		switchButton.setMinWidth(45.0);
		switchButton.setPrefWidth(45.0);
		
		if(this.compositionVM.compositionIsActiveProperty().get()) {
			switchButton.buttonStateProperty().set(State.ON);
		}
		
		this.compositionVM.compositionIsActiveProperty().addListener((object, oldValue, newValue) -> {
			if(newValue) {
				if(switchButton.buttonStateProperty().get() != State.ON) {
					//switchButton.buttonStateProperty().set(State.ON);
				}
			} else {
				if(switchButton.buttonStateProperty().get() != State.OFF) {
					switchButton.buttonStateProperty().set(State.OFF);
				}	
			}
		});
		
		
		switchButton.buttonStateProperty().addListener((object, oldValue, newValue) -> {
			switch(newValue) {
				case ON:
					this.groupVM.activateComposition(this.compositionVM);
					break;
				case OFF: 
					this.groupVM.deactivateComposition(this.compositionVM);
					break;
				default:
					break;
			}
		});
		
		HBox hBox = new HBox();
		hBox.setAlignment(Pos.CENTER_RIGHT);
		AnchorPane.setBottomAnchor(hBox, 0.0);
		AnchorPane.setRightAnchor(hBox, 5.0);
		AnchorPane.setTopAnchor(hBox, 0.0);
		
		Image icon1 = new Image("images/Edit.png");
		ImageView imageView1 = new ImageView();
		imageView1.setImage(icon1);
		imageView1.setFitHeight(25.0);
		imageView1.setFitWidth(25.0);
		imageView1.setPickOnBounds(true);
		imageView1.setPreserveRatio(true);
		
		
		Button buttonEditComposition = new Button();
		buttonEditComposition.setMnemonicParsing(false);
		buttonEditComposition.setStyle("-fx-background-color: none");
		buttonEditComposition.setOnAction(event -> {
			try {
				FXMLLoader loader = new FXMLLoader(getClass().getResource("../composition/editDeletePanel/EditDeleteComposition.fxml"));
				Parent editDeleteCompositionPanel = loader.load();
				EditDeleteCompositionController editDeleteCompositionController = loader.getController();
				Scene scene = new Scene(editDeleteCompositionPanel);
				Stage compositionDialog = new Stage();
				editDeleteCompositionController.init(compositionDialog, viewModelHandler);
				editDeleteCompositionController.setGroup(this.groupVM.getGroupName());
				editDeleteCompositionController.setComposition(compositionName);
				compositionDialog.getIcons().add(new Image("images/Kiwi_Logo.png"));
				compositionDialog.setScene(scene);
				compositionDialog.setTitle("Komposition bearbeiten/lschen");
				compositionDialog.showAndWait();
				editDeleteCompositionController = null;
				loader = null;
			} catch (IOException e) {
				e.printStackTrace();
			}
		});
		buttonEditComposition.setGraphic(imageView1);
		
		Image icon2 = new Image("images/Delete.png");
		ImageView imageView2 = new ImageView();
		imageView2.setImage(icon2);
		imageView2.setFitHeight(25.0);
		imageView2.setFitWidth(25.0);
		imageView2.setPickOnBounds(true);
		imageView2.setPreserveRatio(true);
		
		
		Button buttonDeleteComposition = new Button();
		buttonDeleteComposition.setMnemonicParsing(false);
		buttonDeleteComposition.setStyle("-fx-background-color: none");
		buttonDeleteComposition.setOnAction(event -> {
			Alert alert = new Alert(AlertType.CONFIRMATION);
			Stage stage = (Stage) alert.getDialogPane().getScene().getWindow();
			stage.getIcons().add(new Image("images/Kiwi_Logo.png"));
			alert.setTitle("Komposition lschen");
			alert.setHeaderText(String.format("Sind Sie sicher, dass Sie die Komposition %s lschen mchten", this.getLightingMoodName()));
			ButtonType buttonDelete = new ButtonType("Komposition lschen");
			ButtonType buttonCancel = new ButtonType("Abbrechen", ButtonData.CANCEL_CLOSE);
			alert.getButtonTypes().setAll(buttonDelete, buttonCancel);
			Optional<ButtonType> result = alert.showAndWait();
			if (result.orElseThrow() == buttonDelete) {
				this.groupVM.removeComposition(this.compositionVM);
			}
		});
		buttonDeleteComposition.setGraphic(imageView2);
		
		
		vBoxSwitchButton.getChildren().add(switchButton);
		hBox.getChildren().addAll(buttonEditComposition, buttonDeleteComposition);
		buttonArea.getChildren().addAll(vBoxSwitchButton, hBox);
		vBox.getChildren().addAll(lableCompositionName, buttonArea);
		getChildren().addAll(vBox);
		getStyleClass().addAll("list-item", "item-diselected");
		
		this.selected.bind(this.compositionVM.selectedProperty());
		
		this.selected.addListener((object, oldValue, newValue) -> {
			if(newValue) {
				this.getStyleClass().add("item-selected"); 
				this.getStyleClass().remove("item-diselected");
			} else {
				this.getStyleClass().add("item-diselected");
				this.getStyleClass().remove("item-selected");
			}
		});
		
		this.setOnMouseClicked(event -> {
			this.compositionVM.setCompositionIsSeleceted(true);
		});
	}
	
	public boolean isSelected() {
		return selected.get();
	}
	
	public BooleanProperty selectedProperty() {
		return this.selected;
	}
	
	public String getLightingMoodName() {
		return this.compositionName;
	}
}
