package view.composition.createPanel;

import javafx.fxml.FXML;

import javafx.scene.control.ColorPicker;
import javafx.scene.control.Alert;
import javafx.scene.control.Button;
import javafx.scene.control.TextField;
import javafx.scene.control.ToggleGroup;
import javafx.scene.control.Alert.AlertType;
import javafx.scene.image.Image;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.event.ActionEvent;
import javafx.scene.control.Slider;
import javafx.scene.control.ListView;
import javafx.scene.control.RadioButton;
import javafx.scene.control.ComboBox;
import javafx.scene.control.Spinner;
import javafx.scene.control.SpinnerValueFactory;
import javafx.scene.layout.BorderPane;
import javafx.scene.paint.Color;
import javafx.stage.Stage;
import model.lightingScene.composition.Composition;
import viewModel.KiwiViewModelHandler;
import viewModel.composition.CompositionVM;
import viewModel.fixture.DmxValuesVM;
import viewModel.fixture.FixtureVM;
import viewModel.group.GroupRepositoryVM;
import viewModel.group.GroupVM;
import viewModel.lightingMood.LightingMoodVM;

public class CreateCompositionController {
	@FXML
	private BorderPane root;
	@FXML
	private ColorPicker colorPicker;
	@FXML
	private Slider sliderMainIntensity;
	@FXML
	private Spinner<Integer> spinnerMainIntensity;
	@FXML
	private Slider sliderStrope;
	@FXML
	private Spinner<Integer> spinnerStrope;
	@FXML
	private Slider sliderRed;
	@FXML
	private Spinner<Integer> spinnerRed;
	@FXML
	private Slider sliderGreen;
	@FXML
	private Spinner<Integer> spinnerGreen;
	@FXML
	private Slider sliderBlue;
	@FXML
	private Spinner<Integer> spinnerBlue;
	@FXML
	private Button buttonSave;
	@FXML
	private Button buttonCancel;
	@FXML
	private ComboBox<String> comboBoxGroup;
	@FXML
	private TextField textfieldName;
	@FXML
	private ComboBox<String> comboBoxFixture;
	@FXML
	private ListView<RadioButton> lightMoodList;

	private Stage stage;
	private GroupVM groupVM;
	private FixtureVM fixtureVM;
	private GroupRepositoryVM groupRepositoryVM;
	private ToggleGroup toggelGroup = new ToggleGroup();
	private ObservableList<String> groupNames = FXCollections.observableArrayList();
	private ObservableList<String> fixtureNames = FXCollections.observableArrayList();
	private ObservableList<RadioButton> lightingMoods = FXCollections.observableArrayList();
	
	
	public void init(Stage stage, KiwiViewModelHandler viewModelHandler) {
		this.stage = stage;
		this.groupRepositoryVM = viewModelHandler.getGroupRepositoryVM();
		this.spinnerMainIntensity.setValueFactory(new SpinnerValueFactory.IntegerSpinnerValueFactory(0, 255, 0));
		this.spinnerStrope.setValueFactory(new SpinnerValueFactory.IntegerSpinnerValueFactory(0, 255, 0));
		this.spinnerRed.setValueFactory(new SpinnerValueFactory.IntegerSpinnerValueFactory(0, 255, 0));
		this.spinnerGreen.setValueFactory(new SpinnerValueFactory.IntegerSpinnerValueFactory(0, 255, 0));
		this.spinnerBlue.setValueFactory(new SpinnerValueFactory.IntegerSpinnerValueFactory(0, 255, 0));
		this.lightMoodList.setItems(lightingMoods);
		this.addListener();
		this.loadGroups();
		this.updateLightingMoodList();
		this.loadFixtures();
	}
	
	// Event Listener on Button[#buttonSave].onAction
	@FXML
	public void saveComposition(ActionEvent event) {
		if(this.isDataValid()) {
			
			try {
				CompositionVM compositionVM = new CompositionVM(new Composition(this.textfieldName.getText()));
				this.fixtureNames.forEach(fixtureName -> {
					FixtureVM fixtureVM = this.groupVM.getFixtureRepositoryVM().getFixtureVM(fixtureName);
					compositionVM.setDmxValuesForFixture(fixtureVM.getFixtureName(), fixtureVM.getDmxValuesVM());
				});
				
				this.groupRepositoryVM
					.getGroupVM(this.comboBoxGroup.getValue())
					.addComposition(compositionVM);
				this.stage.close();
			} catch (IllegalArgumentException e) {
				Alert alert = new Alert(AlertType.ERROR);
				Stage stage = (Stage) alert.getDialogPane().getScene().getWindow();
				stage.getIcons().add(new Image("images/Kiwi_Logo.png"));
				alert.setTitle("Fehler");
				alert.setHeaderText("Fehler beim Erstellen einer Lichtstimmung.");
				alert.setContentText(e.getMessage());
				alert.showAndWait();
			}
		}
	}
	// Event Listener on Button[#buttonCancel].onAction
	@FXML
	public void cancel(ActionEvent event) {
		this.stage.close();
	}
	// Event Listener on ComboBox[#comboBoxGroup].onAction
	@FXML
	public void selectGroup(ActionEvent event) {
		if(!(this.comboBoxGroup.getValue() == null)) {
			this.groupVM = groupRepositoryVM.getGroupVM(this.comboBoxGroup.getValue());
			this.updateLightingMoodList();
			this.loadFixtures();
		}
	}
	// Event Listener on ComboBox[#comboBoxFixture].onAction
	@FXML
	public void selectFixture(ActionEvent event) {
		if(!(this.comboBoxFixture.getValue() == null)) {
			this.fixtureVM = this.groupVM.getFixtureRepositoryVM().getFixtureVM(this.comboBoxFixture.getValue());
			this.lightMoodList.getItems().forEach(radioButton -> {
				if(radioButton.getText().equals(fixtureVM.getLightingMoodName())) {
					this.updateSliderDmxValues(this.fixtureVM.getDmxValuesVM());
					radioButton.selectedProperty().set(true);
				}
			});
		}
	}
	
	private void addListener() {
		
		this.addSpinnerListener(this.spinnerMainIntensity, this.sliderMainIntensity);
		this.addSpinnerListener(this.spinnerStrope, this.sliderStrope);
		this.addSpinnerListener(this.spinnerRed, this.sliderRed);
		this.addSpinnerListener(this.spinnerGreen, this.sliderGreen);
		this.addSpinnerListener(this.spinnerBlue, this.sliderBlue);
		
		this.addSliderListener(this.spinnerMainIntensity, this.sliderMainIntensity);
		this.addSliderListener(this.spinnerStrope, this.sliderStrope);
		this.addSliderListener(this.spinnerRed, this.sliderRed);
		this.addSliderListener(this.spinnerGreen, this.sliderGreen);
		this.addSliderListener(this.spinnerBlue, this.sliderBlue);
		
		this.colorPicker.valueProperty().addListener((onject, oldValue, newValue) -> {
			this.sliderRed.setValue(newValue.getRed()*255);
			this.sliderGreen.setValue(newValue.getGreen()*255);
			this.sliderBlue.setValue(newValue.getBlue()*255);
		});
		
		this.sliderRed.valueProperty().addListener((object, oldValue, newValue) -> {
			this.colorPicker
				.valueProperty()
				.setValue(
					new Color(
						(double)newValue/255, 
						this.sliderGreen.getValue()/255, 
						this.sliderBlue.getValue()/255, 1
						));
		});
		
		this.sliderGreen.valueProperty().addListener((object, oldValue, newValue) -> {
			this.colorPicker
				.valueProperty()
				.setValue(
					new Color(
						this.sliderRed.getValue()/255, 
						(double)newValue/255, 
						this.sliderBlue.getValue()/255, 1
						));
		});
		
		this.sliderBlue.valueProperty().addListener((object, oldValue, newValue) -> {
			this.colorPicker
				.valueProperty()
				.setValue(
					new Color(
						this.sliderRed.getValue()/255, 
						this.sliderGreen.getValue()/255, 
						(double)newValue/255, 1
						));
		});
	}
	
	private void addSpinnerListener (Spinner<Integer> spinner, Slider slider) {
		spinner.getValueFactory().valueProperty().addListener((object, oldValue, newValue) -> {
			slider.valueProperty().set(newValue.doubleValue());
		});
	}
	
	private void addSliderListener (Spinner<Integer> spinner, Slider slider) {
		slider.valueProperty().addListener((object, oldValue, newValue) -> {
			if(this.toggelGroup.getSelectedToggle() != null 
					&& this.fixtureVM.getLightingMoodName().equals(((RadioButton)toggelGroup.getSelectedToggle()).getText())) {
				this.updateFixtureDmxValues();
				if(!this.toggelGroup.getToggles().get(0).selectedProperty().get()) {
					this.toggelGroup.getToggles().get(0).selectedProperty().set(true);
				}
			}
			spinner.getValueFactory().setValue(newValue.intValue());
		});
	}
	
	private void loadGroups() {
		this.groupNames.clear();
		this.groupRepositoryVM.groupsListProperty().forEach(groupVM -> {
			this.groupNames.add(groupVM.getGroupName());
		});
		this.groupVM = this.groupRepositoryVM.getGroupVM(this.groupNames.get(0));
		this.comboBoxGroup.setItems(this.groupNames);
		this.comboBoxGroup.setValue(this.groupNames.get(0));
	}

	private void loadFixtures() {
		this.fixtureNames.clear();
		this.groupVM.getFixtureRepositoryVM().fixtureListProperty().forEach(fixtureVM -> {
			this.fixtureNames.add(fixtureVM.getFixtureName());
		});
		this.fixtureVM = this.groupVM.getFixtureRepositoryVM().getFixtureVM(this.fixtureNames.get(0));
		this.comboBoxFixture.setItems(this.fixtureNames);
		this.comboBoxFixture.setValue(this.fixtureNames.get(0));
		this.lightMoodList.getItems().forEach(radioButton -> {
			if(radioButton.getText().equals(this.fixtureVM.getLightingMoodName())) {
				this.updateSliderDmxValues(this.fixtureVM.getDmxValuesVM());
				radioButton.selectedProperty().set(true);
			}
		});
		
	}

	private void updateLightingMoodList() {
		this.lightingMoods.clear();
		this.toggelGroup = new ToggleGroup();
		this.toggelGroup.selectedToggleProperty().addListener((object, oldValue, newValue) -> {
			if(this.fixtureVM != null) {
				RadioButton radioButton = (RadioButton) newValue;
				if(!radioButton.getText().equals("Benutzerdefeniert")) {
					LightingMoodVM lightingMoodVM = this.groupVM.getLightingMoodRepositoryVM().getLightingMood(radioButton.getText());
					this.updateSliderDmxValues(lightingMoodVM.getDmxValuesVM());
					this.updateFixtureDmxValues(lightingMoodVM.getDmxValuesVM());
					this.fixtureVM.setLightingMoodName(radioButton.getText());
				} else {
					this.fixtureVM.setLightingMoodName(radioButton.getText());
				}
			}
		});
		
		RadioButton userSpezified = new RadioButton("Benutzerdefeniert");
		userSpezified.setToggleGroup(toggelGroup);
		this.lightingMoods.add(userSpezified);
		this.groupVM.getLightingMoodRepositoryVM().lightingMoodListProperty().forEach(lightingMood -> {
			RadioButton lightingMoodRadioButton = new RadioButton(lightingMood.getLightingMoodName());
			lightingMoodRadioButton.setToggleGroup(toggelGroup);
			this.lightingMoods.add(lightingMoodRadioButton);
		});
	}
	
	private void updateSliderDmxValues(DmxValuesVM dmxValuesVM) {
		this.sliderMainIntensity.valueProperty().set(dmxValuesVM.getMainIntensity());
		this.sliderStrope.valueProperty().set(dmxValuesVM.getStrope());
		this.sliderRed.valueProperty().set(dmxValuesVM.getRed());
		this.sliderGreen.valueProperty().set(dmxValuesVM.getGreen());
		this.sliderBlue.valueProperty().set(dmxValuesVM.getBlue());
	}

	private void updateFixtureDmxValues() {
		this.fixtureVM.getDmxValuesVM().setMainIntensity(this.sliderMainIntensity.valueProperty().getValue().intValue());
		this.fixtureVM.getDmxValuesVM().setStrope(this.sliderStrope.valueProperty().getValue().intValue());
		this.fixtureVM.getDmxValuesVM().setRed(this.sliderRed.valueProperty().getValue().intValue());
		this.fixtureVM.getDmxValuesVM().setGreen(this.sliderGreen.valueProperty().getValue().intValue());
		this.fixtureVM.getDmxValuesVM().setBlue(this.sliderBlue.valueProperty().getValue().intValue());
	}
	
	private void updateFixtureDmxValues(DmxValuesVM dmxValuesVM) {
		this.fixtureVM.getDmxValuesVM().setMainIntensity(dmxValuesVM.getMainIntensity());
		this.fixtureVM.getDmxValuesVM().setStrope(dmxValuesVM.getStrope());
		this.fixtureVM.getDmxValuesVM().setRed(dmxValuesVM.getRed());
		this.fixtureVM.getDmxValuesVM().setGreen(dmxValuesVM.getGreen());
		this.fixtureVM.getDmxValuesVM().setBlue(dmxValuesVM.getBlue());
	}

	private boolean isDataValid() {
		boolean isValid = false;
		String headerText = "";
		String contentText = "";
		
		if(this.textfieldName.getText().isEmpty()) {
			headerText = "Fehlerhafter Name.";
			contentText = "Das Namensfeld darf nicht leer sein.";
		} else {
			isValid = true;
		}
		
		if(!isValid) {
			Alert alert = new Alert(AlertType.WARNING);
			Stage stage = (Stage) alert.getDialogPane().getScene().getWindow();
			stage.getIcons().add(new Image("images/Kiwi_Logo.png"));
			alert.setTitle("Warnung");
			alert.setHeaderText(headerText);
			alert.setContentText(contentText);
			alert.showAndWait();
		}
				
		return isValid;
	}

	public void setGroup(String name) {
		this.comboBoxGroup.valueProperty().set(name);
		this.groupVM = groupRepositoryVM.getGroupVM(name);
		this.updateLightingMoodList();
		this.loadFixtures();
	}
}
