package view.elements;

import javafx.beans.binding.Bindings;
import javafx.beans.property.IntegerProperty;
import javafx.beans.property.SimpleIntegerProperty;
import javafx.beans.value.ObservableValue;
import javafx.scene.control.TextField;
import javafx.util.converter.NumberStringConverter;

public class NumberField extends TextField {

	private IntegerProperty number;
	
	public NumberField () {
	    initSpellListener();
	    this.number = new SimpleIntegerProperty(1);
	    Bindings.bindBidirectional(this.textProperty(), this.number, new NumberStringConverter());
	    
	}
	
	public final void initSpellListener() {
	    this.textProperty().addListener((ObservableValue<? extends String> observable, String oldValue, String newValue) -> {
	        if (!newValue.matches("\\d*")) {
	            this.setText(newValue.replaceAll("[^\\d]", ""));/*The comma here "[^\\d,]" can be changed with the dot*/
	            StringBuilder aus = new StringBuilder();
	            aus.append(this.getText());
	            boolean firstPointFound = false;
	
	            for (int i = 0; i < aus.length(); i++) {
	                if (aus.charAt(i) == ',') {/*Change the , with . if you want the . to be the decimal separator*/
	                    if (!firstPointFound) {
	                        firstPointFound = true;
	                    } else {
	                        aus.deleteCharAt(i);
	                    }
	                }
	            }
	            newValue = aus.toString();
	            this.setText(newValue);
	        } else {
	            this.setText(newValue);
	        }
	    });
	}
	
	public int getNumber() {
    	if(!this.getText().isEmpty()) {
    		return Integer.parseInt(this.getText());
    	} else {
    		return 0;
    	}	
    }
	
	public IntegerProperty numberProperty() {
		return this.number;
	}
}
