package view.fixture;

import javafx.beans.property.BooleanProperty;
import javafx.beans.property.IntegerProperty;
import javafx.beans.property.SimpleBooleanProperty;
import javafx.beans.property.SimpleIntegerProperty;
import javafx.beans.property.SimpleStringProperty;
import javafx.beans.property.StringProperty;
import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.scene.control.Label;
import javafx.scene.layout.AnchorPane;
import javafx.scene.layout.HBox;
import javafx.scene.layout.Priority;
import javafx.scene.layout.VBox;
import view.elements.ThreeStageSwitchButton;
import viewModel.fixture.FixtureGroupVM;
import viewModel.group.GroupVM.State;


public class FixturesGroupListItem extends AnchorPane{
		
	private IntegerProperty groupId = new SimpleIntegerProperty();
	private StringProperty groupName = new SimpleStringProperty();
	private BooleanProperty fixtureInGroup = new SimpleBooleanProperty(false);
	private BooleanProperty groupSelected = new SimpleBooleanProperty(false);
	private FixtureGroupVM fixtureGroupVM;
	
	public FixturesGroupListItem(FixtureGroupVM fixtureGroupVM) {
		this.fixtureGroupVM = fixtureGroupVM;
		this.groupId.bind(this.fixtureGroupVM.groupIdProperty());
		this.groupName.bind(this.fixtureGroupVM.groupNameProperty());
		this.fixtureInGroup.bind(this.fixtureGroupVM.fixtureInGroupProperty());
		this.groupSelected.bind(this.fixtureGroupVM.groupSelectedProperty());
		
		// style
		setMaxHeight(50.0);
		setMinHeight(50.0);
		setPrefHeight(50.0);
		VBox.setVgrow(this, Priority.ALWAYS);
		VBox.setMargin(this, new Insets(5.0, 5.0, 5.0, 5.0));
		
		VBox vBox = new VBox();
		vBox.setAlignment(Pos.CENTER_LEFT);
		vBox.setLayoutX(7.0);
		vBox.setLayoutY(2.0);
		vBox.setMaxWidth(100.0);
		AnchorPane.setBottomAnchor(vBox, 1.0);
		AnchorPane.setLeftAnchor(vBox, 6.0);
		AnchorPane.setTopAnchor(vBox, 1.0);
		
		Label lableGroupName = new Label();
		lableGroupName.textProperty().bind(groupName);
		lableGroupName.setWrapText(true);
		lableGroupName.getStyleClass().add("text-color-black");
		
		HBox hBox = new HBox();
		hBox.setAlignment(Pos.CENTER_RIGHT);
		AnchorPane.setBottomAnchor(hBox, 0.0);
		AnchorPane.setRightAnchor(hBox, 10.0);
		AnchorPane.setTopAnchor(hBox, 0.0);
		
		
		ThreeStageSwitchButton switchButton = new ThreeStageSwitchButton();
		switchButton.setMaxHeight(20.0);
		switchButton.setMinHeight(20.0);
		switchButton.setPrefHeight(20.0);
		switchButton.setMaxWidth(45.0);
		switchButton.setMinWidth(45.0);
		switchButton.setPrefWidth(45.0);
		switchButton.buttonStateProperty().set(this.fixtureInGroup.get() ? State.ON : State.OFF);
		this.fixtureInGroup.addListener((object, oldValue, newValue) -> {
			if(newValue) {
				switchButton.buttonStateProperty().setValue(State.ON);
			} else {
				switchButton.buttonStateProperty().setValue(State.OFF);
			}
		});
		
		switchButton.buttonStateProperty().addListener((object, oldValue, newValue) -> {
			if(newValue == State.ON && !this.fixtureInGroup.get()) {
				this.fixtureGroupVM.fixtureInGroupProperty().setValue(true);
			} else if(newValue == State.OFF && this.fixtureInGroup.get()) {
				this.fixtureGroupVM.fixtureInGroupProperty().setValue(false);
			}
		});;
		
		
		
		vBox.getChildren().add(lableGroupName);
		hBox.getChildren().add(switchButton);
		getChildren().addAll(vBox,hBox);
		getStyleClass().addAll("list-item", "item-diselected");
		
		
		this.setOnMouseClicked(event -> {
			if(!this.groupSelected.get()) {
				this.fixtureGroupVM.setGroupSelected(true);
			}
			
		});
		
		this.groupSelected.addListener((object, oldValue, newValue) -> {
			if(newValue) {
				this.getStyleClass().add("item-selected");
				this.getStyleClass().remove("item-diselected");
			} else {
				this.getStyleClass().add("item-diselected");
				this.getStyleClass().remove("item-selected");
			}
		});
		
	}
	
	public byte getGroupId() {
		return (byte) this.groupId.get();
	}
	
	public String getGroupName() {
		return this.groupName.get();
	}
	
	public boolean isGroupSelected() {
		return this.groupSelected.get();
	}
	
	public IntegerProperty groupIdProperty() {
		return this.groupId;
	}
	
	public StringProperty groupNameProperty() {
		return this.groupName;
	}
	
	public BooleanProperty groupSelectedProperty() {
		return this.groupSelected;
	}
}
