package view.fixture.createPanel;

import javafx.fxml.FXML;
import javafx.scene.control.Alert;
import javafx.scene.control.Button;
import javafx.scene.control.ListView;
import javafx.scene.control.TextField;
import javafx.scene.control.TextInputDialog;
import javafx.scene.control.Alert.AlertType;
import javafx.scene.image.Image;

import java.util.Optional;

import javafx.collections.FXCollections;
import javafx.collections.ListChangeListener;
import javafx.collections.ObservableList;
import javafx.event.ActionEvent;
import javafx.scene.layout.BorderPane;
import javafx.stage.Stage;
import view.elements.NumberField;
import view.fixture.FixtureGroupListCell;
import view.fixture.FixturesGroupListItem;
import viewModel.KiwiViewModelHandler;
import viewModel.fixture.FixtureGroupRepositoryVM;
import viewModel.fixture.FixtureGroupVM;
import viewModel.fixture.FixtureRepositoryVM;
import viewModel.fixture.FixtureVM;

public class CreateFixtureController {
	@FXML
	private BorderPane root;
	@FXML
	private TextField textboxName;
	@FXML
	private NumberField numberboxAdress;
	@FXML
	private Button buttonCreateGroup;
	@FXML
	private ListView<FixturesGroupListItem> groupList;
	@FXML
	private Button buttonCreateLamp;
	@FXML
	private Button buttonCancel;
	
	private Stage stage;
	private FixtureRepositoryVM fixtureRepositoryVM;
	private FixtureGroupRepositoryVM fixtureGroupRepositoryVM;
	private ObservableList<FixturesGroupListItem> groups = FXCollections.observableArrayList();
	
	public void init(Stage stage, KiwiViewModelHandler viewModelHandler) {
		this.stage = stage;
		this.stage.getIcons().add(new Image("images/Kiwi_Logo.png"));
		this.fixtureRepositoryVM = viewModelHandler.getFixtureRepositoryVM();
		this.fixtureGroupRepositoryVM = viewModelHandler.getFixtureGroupRepositoryVM();
		this.loadGroups();
		
		this.fixtureGroupRepositoryVM.fixtureGroupsListProperty().addListener((ListChangeListener.Change<? extends FixtureGroupVM> change) -> {
			while(change.next()) {
				change.getAddedSubList().forEach(fixtureGroupVM -> {
					this.groups.add(new FixturesGroupListItem(fixtureGroupVM));
				});
				
				change.getRemoved().forEach(groupVM -> {
					int groupListIndex = -1;
    				for(int i = 0; i < this.groups.size(); i++) {
    					if(this.groups.get(i).getGroupName().equals(groupVM.getGroupName())) {
    						groupListIndex = i;
    						break;
    					}
    				}
    				if(groupListIndex != -1) {
    					this.groups.remove(groupListIndex);
    				}
				});	
			}
		});
		
		this.groupList.setItems(this.groups);
		this.groupList.setCellFactory(param -> new FixtureGroupListCell());
		
	}
	
	// Event Listener on Button[#buttonCreateGroup].onAction
	@FXML
	public void createGroup(ActionEvent event) {
		TextInputDialog dialog = new TextInputDialog("Neue Gruppe");
		dialog.setTitle("Neue Gruppe");
		dialog.setHeaderText("Neue Gruppe erstellen");
		dialog.setContentText("Bitte geben Sie den Namen der neuen Gruppe ein:");
		Optional<String> result = dialog.showAndWait();
		result.ifPresent(name -> {
			try {
				this.fixtureGroupRepositoryVM.newGroup(name);
			} catch (IllegalArgumentException e) {
				Alert alert = new Alert(AlertType.ERROR);
				Stage stage = (Stage) alert.getDialogPane().getScene().getWindow();
				stage.getIcons().add(new Image("images/Kiwi_Logo.png"));
				alert.setTitle("Fehler");
				alert.setHeaderText("Fehler beim Erstellen einer Lampe.");
				alert.setContentText(e.getMessage());
				alert.showAndWait();
			}
			
		});
	}
	// Event Listener on Button[#buttonCreateLamp].onAction
	@FXML
	public void createLamp(ActionEvent event) {
		if(this.isDataValid()) {
			try {
				this.fixtureRepositoryVM.newFixture(this.textboxName.getText(), this.numberboxAdress.getNumber());
				FixtureVM fixtureVM = this.fixtureRepositoryVM.getFixtureVM(this.textboxName.getText());
				this.fixtureGroupRepositoryVM.fixtureGroupsListProperty().forEach( fixtureGroup -> {
					if(fixtureGroup.isFixtureInGroup()) {
						fixtureGroup.addFixtureToGroup(fixtureVM);
					}
				});
				
				
				this.stage.close();
			} catch (IllegalArgumentException e) {
				Alert alert = new Alert(AlertType.ERROR);
				alert.setTitle("Fehler");
				Stage stage = (Stage) alert.getDialogPane().getScene().getWindow();
				stage.getIcons().add(new Image("images/Kiwi_Logo.png"));
				alert.setHeaderText("Fehler beim Erstellen einer Lampe.");
				alert.setContentText(e.getMessage());
				alert.showAndWait();
			}
		}
		
	}
	// Event Listener on Button[#buttonCancel].onAction
	@FXML
	public void cancel(ActionEvent event) {
		this.stage.close();
	}
	
	private boolean isDataValid() {
		boolean isValid = false;
		String headerText = "";
		String contentText = "";
		
		if(this.textboxName.getText().isEmpty()) {
			headerText = "Fehlerhafter Name.";
			contentText = "Das Namensfeld darf nicht leer sein.";
		} else if(this.numberboxAdress.getText().isEmpty()) {
			headerText = "Fehlerhafte Adresse.";
			contentText = "Das Adressefeld darf nicht leer sein.";
		} else if(this.numberboxAdress.getNumber() > 512 || this.numberboxAdress.getNumber() < 1) {
			headerText = "Fehlerhafte Adresse.";
			contentText = "Das Adresse der Lampe muss ziwschen 1 und 512 liegen.";
		} else {
			isValid = true;
		}
		
		if(!isValid) {
			Alert alert = new Alert(AlertType.WARNING);
			alert.setTitle("Warnung");
			Stage stage = (Stage) alert.getDialogPane().getScene().getWindow();
			stage.getIcons().add(new Image("images/Kiwi_Logo.png"));
			alert.setHeaderText(headerText);
			alert.setContentText(contentText);
			alert.showAndWait();
		}
				
		return isValid;
	}
	
	public void loadGroups() {
		this.fixtureGroupRepositoryVM.fixtureGroupsListProperty().forEach(fixtureGroupVM -> {
			this.groups.add(new FixturesGroupListItem(fixtureGroupVM));
    	});
	}
}
