package view.group.createPanel;

import javafx.fxml.FXML;
import javafx.scene.control.Alert;
import javafx.scene.control.Button;

import javafx.scene.control.TextField;
import javafx.scene.control.Alert.AlertType;
import javafx.scene.image.Image;
import javafx.collections.FXCollections;
import javafx.collections.ListChangeListener;
import javafx.collections.ObservableList;
import javafx.event.ActionEvent;

import javafx.scene.control.ListView;

import javafx.scene.layout.BorderPane;
import javafx.stage.Stage;
import view.fixture.FixtureListCell;
import view.fixture.FixturesListItem;
import viewModel.KiwiViewModelHandler;
import viewModel.fixture.FixtureRepositoryVM;
import viewModel.fixture.FixtureVM;
import viewModel.group.GroupRepositoryVM;
import viewModel.group.GroupVM;

public class CreateGroupController {
	@FXML
	private BorderPane root;
	@FXML
	private TextField textboxName;
	@FXML
	private ListView<FixturesListItem> groupList;
	@FXML
	private Button buttonCreateGroup;
	@FXML
	private Button buttonCancel;
	
	private FixtureRepositoryVM fixtureRepositoryVM;
	private GroupRepositoryVM groupRepositoryVM;
	private ObservableList<FixturesListItem> fixtures = FXCollections.observableArrayList();
	private Stage stage;
	
	public void init(Stage stage, KiwiViewModelHandler viewModelHandler) {
		this.stage = stage;
		this.fixtureRepositoryVM = viewModelHandler.getFixtureRepositoryVM();
		this.groupRepositoryVM = viewModelHandler.getGroupRepositoryVM();
		this.loadFixtures();
		
		this.fixtureRepositoryVM.fixtureListProperty().addListener((ListChangeListener.Change<? extends FixtureVM> change) -> {
			while(change.next()) {
				change.getAddedSubList().forEach(fixtureVM -> {
					this.fixtures.add(new FixturesListItem(fixtureVM));
				});
				
				change.getRemoved().forEach(fixtureVM -> {
					int fixtureListIndex = -1;
    				for(int i = 0; i < this.fixtures.size(); i++) {
    					if(this.fixtures.get(i).getFixtureName().equals(fixtureVM.getFixtureName())) {
    						fixtureListIndex = i;
    						break;
    					}
    				}
    				if(fixtureListIndex != -1) {
    					this.fixtures.remove(fixtureListIndex);
    				}
				});	
			}
		});
		
		
		this.groupList.setItems(this.fixtures);
		this.groupList.setCellFactory(param -> new FixtureListCell());	
	}
	
	private boolean isDataValid() {
		boolean isValid = false;
		String headerText = "";
		String contentText = "";
		
		if(this.textboxName.getText().isEmpty()) {
			headerText = "Fehlerhafter Name.";
			contentText = "Das Namensfeld darf nicht leer sein.";
		} else {
			isValid = true;
		}
		
		if(!isValid) {
			Alert alert = new Alert(AlertType.WARNING);
			Stage stage = (Stage) alert.getDialogPane().getScene().getWindow();
			stage.getIcons().add(new Image("images/Kiwi_Logo.png"));
			alert.setTitle("Warnung");
			alert.setHeaderText(headerText);
			alert.setContentText(contentText);
			alert.showAndWait();
		}
				
		return isValid;
	}
	
	// Event Listener on Button[#buttonCreateGroup].onAction
	@FXML
	public void createGroup(ActionEvent event) {
		if(isDataValid()) {
			try {
				this.groupRepositoryVM.newGroup(this.textboxName.getText());
				GroupVM groupVM = this.groupRepositoryVM.getGroupVM(this.textboxName.getText());
				this.fixtures.forEach(fixtureListItem -> {
					if(fixtureListItem.isFixtureInGroup()) {
						groupVM.addFixture(fixtureListItem.getFixtureVM());
					}
				});
				this.stage.close();
			} catch (IllegalArgumentException e) {
				Alert alert = new Alert(AlertType.ERROR);
				Stage stage = (Stage) alert.getDialogPane().getScene().getWindow();
				stage.getIcons().add(new Image("images/Kiwi_Logo.png"));
				alert.setTitle("Fehler");
				alert.setHeaderText("Fehler beim Erstellen einer Gruppe.");
				alert.setContentText(e.getMessage());
				alert.showAndWait();
			}
		}
	}
	// Event Listener on Button[#buttonCancel].onAction
	@FXML
	public void cancel(ActionEvent event) {
		this.stage.close();
	}
	
	public void loadFixtures() {
		this.fixtureRepositoryVM.fixturesRepositoryProperty().forEach(fixtureVM -> {
			this.fixtures.add(new FixturesListItem(fixtureVM));
    	});
	}

}
