package view.group.editPanel;

import javafx.fxml.FXML;
import javafx.scene.control.Alert;
import javafx.scene.control.Button;
import javafx.scene.control.ButtonType;
import javafx.scene.control.TextField;
import javafx.scene.control.Alert.AlertType;
import javafx.scene.control.ButtonBar.ButtonData;
import javafx.scene.image.Image;

import java.util.Optional;

import javafx.collections.FXCollections;
import javafx.collections.ListChangeListener;
import javafx.collections.ObservableList;
import javafx.event.ActionEvent;

import javafx.scene.control.ListView;

import javafx.scene.control.ComboBox;

import javafx.scene.layout.BorderPane;
import javafx.stage.Stage;
import view.fixture.FixtureListCell;
import view.fixture.FixturesListItem;
import viewModel.KiwiViewModelHandler;
import viewModel.fixture.FixtureRepositoryVM;
import viewModel.fixture.FixtureVM;
import viewModel.group.GroupRepositoryVM;
import viewModel.group.GroupVM;

public class EditDeleteGroupController {
	@FXML
	private BorderPane root;
	@FXML
	private ComboBox<String> comboboxGroup;
	@FXML
	private Button buttonDelete;
	@FXML
	private TextField textboxName;
	@FXML
	private ListView<FixturesListItem> groupList;
	@FXML
	private Button buttonCreateGroup;
	@FXML
	private Button buttonCancel;
	
	private FixtureRepositoryVM fixtureRepositoryVM;
	private GroupRepositoryVM groupRepositoryVM;
	private ObservableList<FixturesListItem> fixtures = FXCollections.observableArrayList();
	private ObservableList<String> groupNames = FXCollections.observableArrayList();
	private Stage stage;
	private GroupVM groupVM;
	
	public void init(Stage stage, KiwiViewModelHandler viewModelHandler) {
		this.stage = stage;
		this.fixtureRepositoryVM = viewModelHandler.getFixtureRepositoryVM();
		this.groupRepositoryVM = viewModelHandler.getGroupRepositoryVM();
		this.loadGroups();
		this.loadFixtures();
		
		this.fixtureRepositoryVM.fixtureListProperty().addListener((ListChangeListener.Change<? extends FixtureVM> change) -> {
			while(change.next()) {
				change.getAddedSubList().forEach(fixtureVM -> {
					this.fixtures.add(new FixturesListItem(fixtureVM));
				});
				
				change.getRemoved().forEach(fixtureVM -> {
					int fixtureListIndex = -1;
    				for(int i = 0; i < this.fixtures.size(); i++) {
    					if(this.fixtures.get(i).getFixtureName().equals(fixtureVM.getFixtureName())) {
    						fixtureListIndex = i;
    						break;
    					}
    				}
    				if(fixtureListIndex != -1) {
    					this.fixtures.remove(fixtureListIndex);
    				}
				});	
			}
		});
		
		
		this.groupList.setItems(this.fixtures);
		this.groupList.setCellFactory(param -> new FixtureListCell());
		this.updateFixtures();
	}
	
	// Event Listener on ComboBox[#comboboxGroup].onAction
	@FXML
	public void selectGroup(ActionEvent event) {
		if(!(this.comboboxGroup.getValue() == null)) {
			this.groupVM = groupRepositoryVM.getGroupVM(this.comboboxGroup.getValue());
			this.textboxName.setText(this.groupVM.getGroupName());
			this.updateFixtures();
		}
	}
	// Event Listener on Button[#buttonDelete].onAction
	@FXML
	public void deleteGroup(ActionEvent event) {
		Alert alert = new Alert(AlertType.CONFIRMATION);
		Stage stage = (Stage) alert.getDialogPane().getScene().getWindow();
		stage.getIcons().add(new Image("images/Kiwi_Logo.png"));
		alert.setTitle("Gruppe lschen");
		alert.setHeaderText(String.format("Sind Sie sicher, dass Sie die Gruppe %s lschen mchten", this.groupVM.getGroupName()));
		ButtonType buttonDelete = new ButtonType("Gruppe lschen");
		ButtonType buttonCancel = new ButtonType("Abbrechen", ButtonData.CANCEL_CLOSE);
		alert.getButtonTypes().setAll(buttonDelete, buttonCancel);
		Optional<ButtonType> result = alert.showAndWait();
		if (result.orElseThrow() == buttonDelete) {
			this.groupRepositoryVM.deleteGroup(this.textboxName.getText());
			this.loadGroups();
		}
	}
	// Event Listener on Button[#buttonCreateGroup].onAction
	@FXML
	public void createGroup(ActionEvent event) {
		if(this.isDataValid()) {
			this.groupVM.setGroupName(this.textboxName.getText());
			
			this.fixtureRepositoryVM.fixtureListProperty().forEach(fixtureVM -> {
				if(this.groupVM.isFixtureInGroup(fixtureVM.getFixtureName()) && !fixtureVM.isFixtureInGroup()) {
					this.groupVM.removeFixture(fixtureVM);
				} else if (!this.groupVM.isFixtureInGroup(fixtureVM.getFixtureName()) && fixtureVM.isFixtureInGroup()){
					this.groupVM.addFixture(fixtureVM);
				}
			});
			this.stage.close();
		}
		
		
	}
	// Event Listener on Button[#buttonCancel].onAction
	@FXML
	public void cancel(ActionEvent event) {
		this.stage.close();
	}
	
	private boolean isDataValid() {
		boolean isValid = false;
		String headerText = "";
		String contentText = "";
		
		if(this.textboxName.getText().isEmpty()) {
			headerText = "Fehlerhafter Name.";
			contentText = "Das Namensfeld darf nicht leer sein.";
		} else if(nameAllreadyExist(this.textboxName.getText())) {
			headerText = "Fehlerhafter Name.";
			contentText = "Der Name ist bereits in Benutzung.";
		} else {
			isValid = true;
		}
		
		if(!isValid) {
			Alert alert = new Alert(AlertType.WARNING);
			Stage stage = (Stage) alert.getDialogPane().getScene().getWindow();
			stage.getIcons().add(new Image("images/Kiwi_Logo.png"));
			alert.setTitle("Warnung");
			alert.setHeaderText(headerText);
			alert.setContentText(contentText);
			alert.showAndWait();
		}
				
		return isValid;
	}
	
	private boolean nameAllreadyExist(String name) {
		boolean nameExist = false;
		if(!this.groupVM.getGroupName().equals(name) &&
			this.groupNames.stream()
				.filter(groupName -> groupName.equals(name))
				.count() > 0){
			nameExist = true;
		}
		
		return nameExist;
	}
	
	private void loadGroups() {
		this.groupNames.clear();
		this.groupRepositoryVM.groupsListProperty().forEach(groupVM -> {
			this.groupNames.add(groupVM.getGroupName());
		});
		this.groupVM = groupRepositoryVM.getGroupVM(this.groupNames.get(0));
		this.comboboxGroup.setItems(this.groupNames);
		this.comboboxGroup.setValue(this.groupNames.get(0));
		this.textboxName.setText(this.groupVM.getGroupName());
	}
	
	public void loadFixtures() {
		this.fixtureRepositoryVM.fixturesRepositoryProperty().forEach(fixtureVM -> {
			this.fixtures.add(new FixturesListItem(fixtureVM));
    	});
	}
	
	public void updateFixtures() {
		this.fixtureRepositoryVM.fixtureListProperty().forEach(fixtureVM -> {
			if(this.groupVM.isFixtureInGroup(fixtureVM.getFixtureName())) {
				fixtureVM.setFixtureInGroup(true);
			} else {
				fixtureVM.setFixtureInGroup(false);
			}
		});
	}
	
	public void setGroup(GroupVM groupVM) {
		comboboxGroup.valueProperty().set(groupVM.getGroupName());
		this.groupVM = groupRepositoryVM.getGroupVM(this.comboboxGroup.getValue());
		this.textboxName.setText(this.groupVM.getGroupName());
		this.updateFixtures();
	}
	
}
