package view.lightingMood;

import java.io.IOException;
import java.util.Optional;

import javafx.beans.property.BooleanProperty;
import javafx.beans.property.SimpleBooleanProperty;
import javafx.fxml.FXMLLoader;
import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.scene.control.Alert;
import javafx.scene.control.Button;
import javafx.scene.control.ButtonType;
import javafx.scene.control.Label;
import javafx.scene.control.Alert.AlertType;
import javafx.scene.control.ButtonBar.ButtonData;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import javafx.scene.layout.AnchorPane;
import javafx.scene.layout.FlowPane;
import javafx.scene.layout.HBox;
import javafx.scene.layout.Priority;
import javafx.scene.layout.VBox;
import javafx.stage.Stage;
import view.elements.ThreeStageSwitchButton;
import view.lightingMood.editDeletePanel.EditDeleteLightingMoodController;
import viewModel.KiwiViewModelHandler;
import viewModel.group.GroupVM;
import viewModel.group.GroupVM.State;
import viewModel.lightingMood.LightingMoodVM;

public class LightingMoodListItem extends AnchorPane{
	
	private LightingMoodVM lightingMoodVM;
	private String lightingMoodName;
	private BooleanProperty selected;
	private GroupVM groupVM;
	
	public LightingMoodListItem(LightingMoodVM lightingMoodVM, GroupVM groupVM, KiwiViewModelHandler viewModelHandler) {
		this.lightingMoodVM = lightingMoodVM;
		this.groupVM = groupVM;
		this.lightingMoodName = lightingMoodVM.getLightingMoodName();
		selected = new SimpleBooleanProperty(false);
		
		setMaxHeight(80.0);
		setMinHeight(80.0);
		setPrefHeight(80.0);
		setMaxWidth(200.0);
		setMinWidth(200.0);
		setPrefWidth(200.0);
		FlowPane.setMargin(this, new Insets(5.0, 5.0, 5.0, 5.0));
		
		VBox vBox = new VBox();
		vBox.setAlignment(Pos.CENTER_LEFT);
		vBox.setLayoutX(7.0);
		vBox.setLayoutY(2.0);
		vBox.setMaxWidth(100.0);
		AnchorPane.setBottomAnchor(vBox, 1.0);
		AnchorPane.setLeftAnchor(vBox, 5.0);
		AnchorPane.setRightAnchor(vBox, 5.0);
		AnchorPane.setTopAnchor(vBox, 1.0);
		
		Label lableLightingMoodName = new Label(lightingMoodName);
		lableLightingMoodName.setAlignment(Pos.TOP_LEFT);
		lableLightingMoodName.setMaxHeight(34.0);
		lableLightingMoodName.setWrapText(true);
		lableLightingMoodName.textProperty().bind(this.lightingMoodVM.LightingMoodNameProperty());
		VBox.setVgrow(lableLightingMoodName, Priority.ALWAYS);
		VBox.setMargin(lableLightingMoodName, new Insets(5.0, 0.0, 0.0, 0.0));
		
		AnchorPane buttonArea = new AnchorPane();
		
		VBox vBoxSwitchButton = new VBox();
		vBoxSwitchButton.setAlignment(Pos.CENTER_LEFT);
		vBoxSwitchButton.setLayoutX(-1.0);
		vBoxSwitchButton.setLayoutY(-1.0);
		AnchorPane.setBottomAnchor(vBoxSwitchButton, 0.0);
		AnchorPane.setLeftAnchor(vBoxSwitchButton, 5.0);
		AnchorPane.setTopAnchor(vBoxSwitchButton, 0.0);
		
		ThreeStageSwitchButton switchButton = new ThreeStageSwitchButton();
		switchButton.setMaxHeight(20.0);
		switchButton.setMinHeight(20.0);
		switchButton.setPrefHeight(20.0);
		switchButton.setMaxWidth(45.0);
		switchButton.setMinWidth(45.0);
		switchButton.setPrefWidth(45.0);
		
		if(this.lightingMoodVM.LightingMoodIsActiveProperty().get()) {
			switchButton.buttonStateProperty().set(State.ON);
		}
		
		this.lightingMoodVM.LightingMoodIsActiveProperty().addListener((object, oldValue, newValue) -> {
			if(newValue) {
				if(switchButton.buttonStateProperty().get() != State.ON) {
					//switchButton.buttonStateProperty().set(State.ON);
				}
			} else {
				if(switchButton.buttonStateProperty().get() != State.OFF) {
					switchButton.buttonStateProperty().set(State.OFF);
				}	
			}
		});
		
		
		switchButton.buttonStateProperty().addListener((object, oldValue, newValue) -> {
			switch(newValue) {
				case ON:
					this.groupVM.activateLightingMood(this.lightingMoodVM);
					break;
				case OFF: 
					this.groupVM.deactivateLightingMood(this.lightingMoodVM);
					break;
				default:
					break;
			}
		});
		
		HBox hBox = new HBox();
		hBox.setAlignment(Pos.CENTER_RIGHT);
		AnchorPane.setBottomAnchor(hBox, 0.0);
		AnchorPane.setRightAnchor(hBox, 5.0);
		AnchorPane.setTopAnchor(hBox, 0.0);
		
		Image icon1 = new Image("images/Edit.png");
		ImageView imageView1 = new ImageView();
		imageView1.setImage(icon1);
		imageView1.setFitHeight(25.0);
		imageView1.setFitWidth(25.0);
		imageView1.setPickOnBounds(true);
		imageView1.setPreserveRatio(true);
		
		
		Button buttonEditGroup = new Button();
		buttonEditGroup.setMnemonicParsing(false);
		buttonEditGroup.setStyle("-fx-background-color: none");
		buttonEditGroup.setOnAction(event -> {
			try {
				FXMLLoader loader = new FXMLLoader(getClass().getResource("./editDeletePanel/EditDeleteLightingMood.fxml"));
				Parent editDeleteLightingMoodPanel;
				editDeleteLightingMoodPanel = loader.load();
				EditDeleteLightingMoodController editDeleteLightingMoodController = loader.getController();
				Scene scene = new Scene(editDeleteLightingMoodPanel);
				Stage lightingMoodDialog = new Stage();
				editDeleteLightingMoodController.init(lightingMoodDialog, viewModelHandler);
				editDeleteLightingMoodController.setGroup(this.groupVM.getGroupName());
				editDeleteLightingMoodController.setLightingMood(this.lightingMoodName);
				lightingMoodDialog.getIcons().add(new Image("images/Kiwi_Logo.png"));
				lightingMoodDialog.setScene(scene);
				lightingMoodDialog.setTitle("Lichtstimmung bearbeiten/lschen");
				lightingMoodDialog.showAndWait();
			} catch (IOException e) {
				e.printStackTrace();
			}
		});
		buttonEditGroup.setGraphic(imageView1);
		
		Image icon2 = new Image("images/Delete.png");
		ImageView imageView2 = new ImageView();
		imageView2.setImage(icon2);
		imageView2.setFitHeight(25.0);
		imageView2.setFitWidth(25.0);
		imageView2.setPickOnBounds(true);
		imageView2.setPreserveRatio(true);
		
		
		Button buttonDeleteGroup = new Button();
		buttonDeleteGroup.setMnemonicParsing(false);
		buttonDeleteGroup.setStyle("-fx-background-color: none");
		buttonDeleteGroup.setOnAction(event -> {
			Alert alert = new Alert(AlertType.CONFIRMATION);
			Stage stage = (Stage) alert.getDialogPane().getScene().getWindow();
			stage.getIcons().add(new Image("images/Kiwi_Logo.png"));
			alert.setTitle("Lichtstimmung lschen");
			alert.setHeaderText(String.format("Sind Sie sicher, dass Sie die Lichtstimmung %s lschen mchten", this.getLightingMoodName()));
			ButtonType buttonDelete = new ButtonType("Lichstimmung lschen");
			ButtonType buttonCancel = new ButtonType("Abbrechen", ButtonData.CANCEL_CLOSE);
			alert.getButtonTypes().setAll(buttonDelete, buttonCancel);
			Optional<ButtonType> result = alert.showAndWait();
			if (result.orElseThrow() == buttonDelete) {
				this.groupVM.removeLightingMood(this.lightingMoodVM);
			}
		});
		buttonDeleteGroup.setGraphic(imageView2);
		
		
		vBoxSwitchButton.getChildren().add(switchButton);
		hBox.getChildren().addAll(buttonEditGroup, buttonDeleteGroup);
		buttonArea.getChildren().addAll(vBoxSwitchButton, hBox);
		vBox.getChildren().addAll(lableLightingMoodName, buttonArea);
		getChildren().addAll(vBox);
		getStyleClass().addAll("list-item", "item-diselected");
		
		this.selected.bind(this.lightingMoodVM.LightingMoodIsSelectedProperty());
		
		this.selected.addListener((object, oldValue, newValue) -> {
			if(newValue) {
				this.getStyleClass().add("item-selected"); 
				this.getStyleClass().remove("item-diselected");
			} else {
				this.getStyleClass().add("item-diselected");
				this.getStyleClass().remove("item-selected");
			}
		});
		
		this.setOnMouseClicked(event -> {
			this.lightingMoodVM.setLightingMoodIsSelected(true);
		});
	}
	
	public boolean isSelected() {
		return selected.get();
	}
	
	public BooleanProperty selectedProperty() {
		return this.selected;
	}
	
	public String getLightingMoodName() {
		return this.lightingMoodName;
	}
}
