package view.lightingMood.editDeletePanel;

import javafx.fxml.FXML;

import javafx.scene.control.ColorPicker;
import javafx.scene.control.Alert;
import javafx.scene.control.Button;
import javafx.scene.control.ButtonType;
import javafx.scene.control.TextField;
import javafx.scene.control.Alert.AlertType;
import javafx.scene.control.ButtonBar.ButtonData;
import javafx.scene.image.Image;

import java.util.Optional;

import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.event.ActionEvent;

import javafx.scene.control.Slider;

import javafx.scene.control.ComboBox;

import javafx.scene.control.Spinner;
import javafx.scene.control.SpinnerValueFactory;
import javafx.scene.layout.BorderPane;
import javafx.scene.paint.Color;
import javafx.stage.Stage;
import model.lightingScene.lightingMood.LightingMood;
import viewModel.KiwiViewModelHandler;
import viewModel.group.GroupRepositoryVM;
import viewModel.group.GroupVM;
import viewModel.lightingMood.LightingMoodRepositoryVM;
import viewModel.lightingMood.LightingMoodVM;

public class EditDeleteLightingMoodController {
	@FXML
	private BorderPane root;
	@FXML
	private Slider sliderMainIntensity;
	@FXML
	private Spinner<Integer> spinnerMainIntensity;
	@FXML
	private Slider sliderStrope;
	@FXML
	private Spinner<Integer> spinnerStrope;
	@FXML
	private ColorPicker colorPicker;
	@FXML
	private Slider sliderRed;
	@FXML
	private Spinner<Integer> spinnerRed;
	@FXML
	private Slider sliderGreen;
	@FXML
	private Spinner<Integer> spinnerGreen;
	@FXML
	private Slider sliderBlue;
	@FXML
	private Spinner<Integer> spinnerBlue;
	@FXML
	private ComboBox<String> comboBoxLightingMood;
	@FXML
	private ComboBox<String> comboBoxGroup;
	@FXML
	private TextField textfieldName;
	@FXML
	private Button buttonSave;
	@FXML
	private Button buttonCancel;
	@FXML
	private Button buttonDelete;

	private Stage stage;
	private ObservableList<String> groupNames = FXCollections.observableArrayList();
	private ObservableList<String> lightingMoodNames = FXCollections.observableArrayList();
	private GroupRepositoryVM groupRepositoryVM;
	private LightingMoodRepositoryVM lightingMoodRepositoryVM;
	private GroupVM groupVM;
	private LightingMoodVM lightingMoodVM;
	
	public void init(Stage stage, KiwiViewModelHandler viewModelHandler) {
		this.stage = stage;
		this.groupRepositoryVM = viewModelHandler.getGroupRepositoryVM();
		this.spinnerMainIntensity.setValueFactory(new SpinnerValueFactory.IntegerSpinnerValueFactory(0, 255, 255));
		this.spinnerStrope.setValueFactory(new SpinnerValueFactory.IntegerSpinnerValueFactory(0, 255, 0));
		this.spinnerRed.setValueFactory(new SpinnerValueFactory.IntegerSpinnerValueFactory(0, 255, 255));
		this.spinnerGreen.setValueFactory(new SpinnerValueFactory.IntegerSpinnerValueFactory(0, 255, 255));
		this.spinnerBlue.setValueFactory(new SpinnerValueFactory.IntegerSpinnerValueFactory(0, 255, 255));
		this.addListener();
		this.loadGroups();
	}
	
	// Event Listener on ComboBox[#comboBoxGroup].onAction
	@FXML
	public void selectGroup(ActionEvent event) {
		if(!(this.comboBoxGroup.getValue() == null)) {
			this.groupVM = groupRepositoryVM.getGroupVM(this.comboBoxGroup.getValue());
			this.loadLightingMoods();	
		}
	}
	
	// Event Listener on ComboBox[#comboBoxLightingMood].onAction
	@FXML
	public void selectLightingMood(ActionEvent event) {
		if(!(this.comboBoxLightingMood.getValue() == null)) {
			this.lightingMoodVM = lightingMoodRepositoryVM.getLightingMood(this.comboBoxLightingMood.getValue());
			this.updateLightingMood();
		}
	}
	
	// Event Listener on Button[#buttonDelete].onAction
	public void deleteLightingMood(ActionEvent event) {
		Alert alert = new Alert(AlertType.CONFIRMATION);
		Stage stage = (Stage) alert.getDialogPane().getScene().getWindow();
		stage.getIcons().add(new Image("images/Kiwi_Logo.png"));
		alert.setTitle("Lichtstimmung lschen");
		alert.setHeaderText(String.format("Sind Sie sicher, dass Sie die Lichtstimmung %s lschen mchten", this.lightingMoodVM.getLightingMoodName()));
		ButtonType buttonDelete = new ButtonType("Lichtstimmung lschen");
		ButtonType buttonCancel = new ButtonType("Abbrechen", ButtonData.CANCEL_CLOSE);
		alert.getButtonTypes().setAll(buttonDelete, buttonCancel);
		Optional<ButtonType> result = alert.showAndWait();
		if (result.orElseThrow() == buttonDelete) {
			this.lightingMoodRepositoryVM.removeLightingMood(lightingMoodVM);
			this.loadLightingMoods();
		}
	}
	
	// Event Listener on Button[#buttonSave].onAction
	@FXML
	public void saveLightingMood(ActionEvent event) {
		if(isDataValid()) {
			this.lightingMoodVM.setLightingMoodName(this.textfieldName.getText());
			this.lightingMoodVM.setLightingMoodMainIntensity(this.sliderMainIntensity.valueProperty().getValue().intValue());
			this.lightingMoodVM.setLightingMoodStrope(this.sliderStrope.valueProperty().getValue().intValue());
			this.lightingMoodVM.setLightingMoodRed(this.sliderRed.valueProperty().getValue().intValue());
			this.lightingMoodVM.setLightingMoodGreen(this.sliderGreen.valueProperty().getValue().intValue());
			this.lightingMoodVM.setLightingMoodBlue(this.sliderBlue.valueProperty().getValue().intValue());
			this.stage.close();
		}
	}
	
	// Event Listener on Button[#buttonCancel].onAction
	@FXML
	public void cancel(ActionEvent event) {
		this.stage.close();
	}
	
	private void addListener() {
		this.addSpinnerListener(this.spinnerMainIntensity, this.sliderMainIntensity);
		this.addSpinnerListener(this.spinnerStrope, this.sliderStrope);
		this.addSpinnerListener(this.spinnerRed, this.sliderRed);
		this.addSpinnerListener(this.spinnerGreen, this.sliderGreen);
		this.addSpinnerListener(this.spinnerBlue, this.sliderBlue);
		
		this.addSliderListener(this.spinnerMainIntensity, this.sliderMainIntensity);
		this.addSliderListener(this.spinnerStrope, this.sliderStrope);
		this.addSliderListener(this.spinnerRed, this.sliderRed);
		this.addSliderListener(this.spinnerGreen, this.sliderGreen);
		this.addSliderListener(this.spinnerBlue, this.sliderBlue);
		
		this.colorPicker.valueProperty().addListener((onject, oldValue, newValue) -> {
			this.sliderRed.setValue(newValue.getRed()*255);
			this.sliderGreen.setValue(newValue.getGreen()*255);
			this.sliderBlue.setValue(newValue.getBlue()*255);
		});
		
		this.sliderRed.valueProperty().addListener((object, oldValue, newValue) -> {
			this.colorPicker
				.valueProperty()
				.setValue(
					new Color(
						(double)newValue/255, 
						this.sliderGreen.getValue()/255, 
						this.sliderBlue.getValue()/255, 1
						));
		});
		
		this.sliderGreen.valueProperty().addListener((object, oldValue, newValue) -> {
			this.colorPicker
				.valueProperty()
				.setValue(
					new Color(
						this.sliderRed.getValue()/255, 
						(double)newValue/255, 
						this.sliderBlue.getValue()/255, 1
						));
		});
		
		this.sliderBlue.valueProperty().addListener((object, oldValue, newValue) -> {
			this.colorPicker
				.valueProperty()
				.setValue(
					new Color(
						this.sliderRed.getValue()/255, 
						this.sliderGreen.getValue()/255, 
						(double)newValue/255, 1
						));
		});
	}
	
	private void addSpinnerListener (Spinner<Integer> spinner, Slider slider) {
		spinner.getValueFactory().valueProperty().addListener((object, oldValue, newValue) -> {
			slider.valueProperty().set(newValue.doubleValue());
		});
	}
	
	private void addSliderListener (Spinner<Integer> spinner, Slider slider) {
		slider.valueProperty().addListener((object, oldValue, newValue) -> {
			spinner.getValueFactory().setValue(newValue.intValue());
		});
	}
	
	private void loadGroups() {
		this.groupNames.clear();
		this.groupRepositoryVM.groupsListProperty().forEach(groupVM -> {
			this.groupNames.add(groupVM.getGroupName());
		});
		this.groupVM = groupRepositoryVM.getGroupVM(this.groupNames.get(0));
		this.comboBoxGroup.setItems(this.groupNames);
		this.comboBoxGroup.setValue(this.groupNames.get(0));
		this.loadLightingMoods();
	}
	
	private void loadLightingMoods() {
		this.lightingMoodNames.clear();
		this.lightingMoodRepositoryVM = groupVM.getLightingMoodRepositoryVM();
		this.lightingMoodRepositoryVM.lightingMoodListProperty().forEach(lightingMoodVM -> {
			this.lightingMoodNames.add(lightingMoodVM.getLightingMoodName());
		});
		this.comboBoxLightingMood.setItems(this.lightingMoodNames);
		if(lightingMoodRepositoryVM.lightingMoodListProperty().size() > 0) {
			this.lightingMoodVM = lightingMoodRepositoryVM.getLightingMood(this.lightingMoodNames.get(0));
			this.comboBoxLightingMood.setValue(this.lightingMoodNames.get(0));
		} else {
			this.lightingMoodVM = new LightingMoodVM(new LightingMood("Lichtstimmung"));
		}
		this.updateLightingMood();
	}
	
	public void updateLightingMood() {
		this.textfieldName.setText(this.lightingMoodVM.getLightingMoodName());
		this.sliderMainIntensity.valueProperty().set(this.lightingMoodVM.getLightingMoodMainIntensity());
		this.sliderStrope.valueProperty().set(this.lightingMoodVM.getLightingMoodStrope());
		this.sliderRed.valueProperty().set(this.lightingMoodVM.getLightingMoodRed());
		this.sliderGreen.valueProperty().set(this.lightingMoodVM.getLightingMoodGreen());
		this.sliderBlue.valueProperty().set(this.lightingMoodVM.getLightingMoodBlue());
	}
	
	private boolean isDataValid() {
		boolean isValid = false;
		String headerText = "";
		String contentText = "";
		
		if(this.textfieldName.getText().isEmpty()) {
			headerText = "Fehlerhafter Name.";
			contentText = "Das Namensfeld darf nicht leer sein.";
		} else if(this.nameAllreadyExist(this.textfieldName.getText())) {
			headerText = "Fehlerhafter Name.";
			contentText = "Der Name ist bereits in Benutzung.";
		} else {
			isValid = true;
		}
		
		if(!isValid) {
			Alert alert = new Alert(AlertType.WARNING);
			alert.setTitle("Warnung");
			alert.setHeaderText(headerText);
			alert.setContentText(contentText);
			alert.showAndWait();
		}
				
		return isValid;
	}
	
	private boolean nameAllreadyExist(String name) {
		boolean nameExist = false;
		if(!this.lightingMoodVM.getLightingMoodName().equals(name) &&
			this.lightingMoodNames.stream()
				.filter(lightingMoodName -> lightingMoodName.equals(name))
				.count() > 0){
			nameExist = true;
		}
		
		return nameExist;
	}

	public void setGroup(String groupName) {
		this.comboBoxGroup.valueProperty().set(groupName);
		this.groupVM = groupRepositoryVM.getGroupVM(this.comboBoxGroup.getValue());
		this.loadLightingMoods();
	}
	
	public void setLightingMood(String lightingMoodName) {
		this.comboBoxLightingMood.valueProperty().set(lightingMoodName);
		this.lightingMoodVM = lightingMoodRepositoryVM.getLightingMood(this.comboBoxLightingMood.getValue());
		this.updateLightingMood();
	}
}
