package view.lightingMood.mainPanel;

import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import java.io.IOException;
import javafx.collections.ListChangeListener;
import javafx.event.ActionEvent;
import javafx.scene.layout.FlowPane;
import javafx.stage.Stage;
import view.lightingMood.LightingMoodListItem;
import view.lightingMood.createPanel.CreateLightingMoodController;
import viewModel.KiwiViewModelHandler;
import viewModel.group.GroupVM;
import viewModel.lightingMood.LightingMoodRepositoryVM;
import viewModel.lightingMood.LightingMoodVM;
import javafx.scene.image.Image;

public class LightingMoodMainPanelController {
	@FXML
	private FlowPane flowPaneLightingMoodList;
	@FXML
	private Button buttonCreateNewLightingMood;

	private KiwiViewModelHandler viewModelHandler;
	private LightingMoodRepositoryVM lightingMoodRepositoryVM;
	private GroupVM groupVM;

	public void init(KiwiViewModelHandler viewModelHandler) {
		this.viewModelHandler = viewModelHandler;
	}
	
	// Event Listener on Button[#buttonCreateNewLightingMood].onAction
	@FXML
	public void createLightingMood(ActionEvent event) throws IOException {
		FXMLLoader loader = new FXMLLoader(getClass().getResource("../createPanel/CreateLightingMood.fxml"));
		Parent createLightingMoodPanel = loader.load();
		CreateLightingMoodController createLightingMoodController = loader.getController();
		Scene scene = new Scene(createLightingMoodPanel);
		Stage lightingMoodDialog = new Stage();
		createLightingMoodController.init(lightingMoodDialog, this.viewModelHandler.getGroupRepositoryVM());
		createLightingMoodController.setGroup(this.groupVM.getGroupName());
		lightingMoodDialog.getIcons().add(new Image("images/Kiwi_Logo.png"));
		lightingMoodDialog.setScene(scene);
		lightingMoodDialog.setTitle("Lichtstimmung erstellen");
		lightingMoodDialog.showAndWait();
	}
	
	private void addLightingMoodToPane(LightingMoodVM lightingMoodVM) {
		this.flowPaneLightingMoodList.getChildren().add(new LightingMoodListItem(lightingMoodVM, groupVM, this.viewModelHandler));
	}
	
	public void setLightingMoodRepository(GroupVM groupVM) {
		this.groupVM = groupVM;
		this.lightingMoodRepositoryVM = groupVM.getLightingMoodRepositoryVM();
		this.flowPaneLightingMoodList.getChildren().clear();
		this.lightingMoodRepositoryVM.lightingMoodListProperty().forEach(lightingMoodVM -> {
			this.addLightingMoodToPane(lightingMoodVM);
		});
		
		this.lightingMoodRepositoryVM.lightingMoodListProperty().addListener((ListChangeListener.Change<? extends LightingMoodVM> change) -> {
			while(change.next()) {
				change.getAddedSubList().forEach(lightingMoodVM -> {
					addLightingMoodToPane(lightingMoodVM);
				});
				
				change.getRemoved().forEach(lightingMoodVM -> {
					int lightingMoodListIndex = -1;
    				for(int i = 0; i < this.flowPaneLightingMoodList.getChildren().size(); i++) {
    					if(((LightingMoodListItem)this.flowPaneLightingMoodList
    							.getChildren()
    							.get(i))
    							.getLightingMoodName()
    							.equals(lightingMoodVM.getLightingMoodName())) {
    						lightingMoodListIndex = i;
    						break;
    					}
    				}
    				if(lightingMoodListIndex != -1) {
    					this.flowPaneLightingMoodList.getChildren().remove(lightingMoodListIndex);
    				}
				});	
			}
		});
	}
	
}
