package view.mainScene;

import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.scene.control.SplitPane;
import javafx.scene.control.Alert.AlertType;
import javafx.scene.image.Image;

import java.awt.Desktop;
import java.io.IOException;
import java.net.URI;

import javafx.application.Platform;
import javafx.event.ActionEvent;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.scene.control.Alert;
import javafx.scene.control.MenuItem;
import javafx.scene.layout.BorderPane;
import javafx.stage.Stage;
import view.composition.createPanel.CreateCompositionController;
import view.composition.editDeletePanel.EditDeleteCompositionController;
import view.composition.mainPanel.CompositionMainPanelController;
import view.fixture.createPanel.CreateFixtureController;
import view.fixture.editPanel.EditDeleteFixtureController;
import view.group.createPanel.CreateGroupController;
import view.group.editPanel.EditDeleteGroupController;
import view.group.mainPanel.GroupMainPanelController;
import view.lightController.mainPanel.LightControllerMainPanelController;
import view.lightingMood.createPanel.CreateLightingMoodController;
import view.lightingMood.editDeletePanel.EditDeleteLightingMoodController;
import view.lightingMood.mainPanel.LightingMoodMainPanelController;
import viewModel.KiwiViewModelHandler;

public class MainSceneController {
	@FXML
	private MenuItem menuFileNew;
	@FXML
	private MenuItem menuFileLoad;
	@FXML
	private MenuItem menuFileSave;
	@FXML
	private MenuItem menuFileSaveAs;
	@FXML
	private MenuItem menuFileQuit;
	@FXML
	private MenuItem menuFixtureAdd;
	@FXML
	private MenuItem menuFixtureEditRemove;
	@FXML
	private BorderPane groupInterfacePanel;
	@FXML
	private SplitPane compositionLightingMoodPanel;
	
	private KiwiViewModelHandler viewModelHandler;
	
	public void init(KiwiViewModelHandler viewModelHandler) throws IOException {
		this.viewModelHandler = viewModelHandler;
		
		FXMLLoader loaderCompositionPanel = new FXMLLoader(getClass().getResource("../composition/mainPanel/CompositionMainPanel.fxml"));
		Parent compositionPanel = loaderCompositionPanel.load();	
		CompositionMainPanelController compositionPanelController = loaderCompositionPanel.getController();
		compositionPanelController.init(viewModelHandler);
		compositionLightingMoodPanel.getItems().add(compositionPanel);
		
		FXMLLoader loaderLightingMoodPanel = new FXMLLoader(getClass().getResource("../lightingMood/mainPanel/LightingMoodMainPanel.fxml"));
		Parent lightingMoodPanel = loaderLightingMoodPanel.load();	
		LightingMoodMainPanelController lightingMoodPanelController = loaderLightingMoodPanel.getController();
		lightingMoodPanelController.init(this.viewModelHandler);
		compositionLightingMoodPanel.getItems().add(lightingMoodPanel);
		
		FXMLLoader loaderGroupPanel = new FXMLLoader(getClass().getResource("../group/mainPanel/GroupMainPanel.fxml"));
		Parent groupPanel = loaderGroupPanel.load();
		GroupMainPanelController groupPanelController = loaderGroupPanel.getController();
		groupPanelController.init(viewModelHandler, lightingMoodPanelController, compositionPanelController);
		groupInterfacePanel.setCenter(groupPanel);
		
		FXMLLoader loaderLightControllerPanel = new FXMLLoader(getClass().getResource("../lightController/mainPanel/LightControllerMainPanel.fxml"));
		Parent lightControllerPanel = loaderLightControllerPanel.load();	
		LightControllerMainPanelController lightControllerPanelController = loaderLightControllerPanel.getController();
		lightControllerPanelController.init(this.viewModelHandler.getArtNetControlleVM());
		groupInterfacePanel.setBottom(lightControllerPanel);
	}
	
	// Event Listener on MenuItem[#menuFileNew].onAction
	@FXML
	public void createFile(ActionEvent event) {
		this.viewModelHandler.clearRepros();
	}
	// Event Listener on MenuItem[#menuFileLoad].onAction
	@FXML
	public void loadFile(ActionEvent event) {
		Alert alert = new Alert(AlertType.WARNING);
		Stage stage = (Stage) alert.getDialogPane().getScene().getWindow();
		stage.getIcons().add(new Image("images/Kiwi_Logo.png"));
		alert.setTitle("Warnung");
		alert.setHeaderText("Funktion nicht verfgbar.");
		alert.setContentText("Die Funktion zum Laden von Files ist aktuell noch nicht verfgbar.");
		alert.showAndWait();
	}
	// Event Listener on MenuItem[#menuFileSave].onAction
	@FXML
	public void saveFile(ActionEvent event) {
		Alert alert = new Alert(AlertType.WARNING);
		Stage stage = (Stage) alert.getDialogPane().getScene().getWindow();
		stage.getIcons().add(new Image("images/Kiwi_Logo.png"));
		alert.setTitle("Warnung");
		alert.setHeaderText("Funktion nicht verfgbar.");
		alert.setContentText("Die Funktion zum Speichern von Files ist aktuell noch nicht verfgbar.");
		alert.showAndWait();
	}
	// Event Listener on MenuItem[#menuFileSaveAs].onAction
	@FXML
	public void saveFileAs(ActionEvent event) {
		Alert alert = new Alert(AlertType.WARNING);
		Stage stage = (Stage) alert.getDialogPane().getScene().getWindow();
		stage.getIcons().add(new Image("images/Kiwi_Logo.png"));
		alert.setTitle("Warnung");
		alert.setHeaderText("Funktion nicht verfgbar.");
		alert.setContentText("Die Funktion zum Speichern von Files ist aktuell noch nicht verfgbar.");
		alert.showAndWait();
	}
	// Event Listener on MenuItem[#menuFileQuit].onAction
	@FXML
	public void quit(ActionEvent event) {
		Platform.exit();
	}
	// Event Listener on MenuItem[#menuFixtureAdd].onAction
	@FXML
	public void createFixtures(ActionEvent event) throws IOException {
		FXMLLoader loader = new FXMLLoader(getClass().getResource("../fixture/createPanel/CreateFixture.fxml"));
		Parent createFixturePanel = loader.load();
		CreateFixtureController createFixtureController = loader.getController();
		Scene scene = new Scene(createFixturePanel);
		Stage fixtureDialog = new Stage();
		createFixtureController.init(fixtureDialog, this.viewModelHandler);
		fixtureDialog.getIcons().add(new Image("images/Kiwi_Logo.png"));
		fixtureDialog.setScene(scene);
		fixtureDialog.setTitle("Lampe hinzufgen");
		fixtureDialog.showAndWait();
	}
	// Event Listener on MenuItem[#menuFixtureEditRemove].onAction
	@FXML
	public void editDeleteFixtures(ActionEvent event) throws IOException {
		if(this.viewModelHandler.getFixtureRepositoryVM().fixturesRepositoryProperty().size() > 0) {
			FXMLLoader loader = new FXMLLoader(getClass().getResource("../fixture/editPanel/EditDeleteFixture.fxml"));
			Parent editDeletefixturePanel = loader.load();
			EditDeleteFixtureController editDeleteFixtureController = loader.getController();
			Scene scene = new Scene(editDeletefixturePanel);
			Stage fixtureDialog = new Stage();
			editDeleteFixtureController.init(fixtureDialog, this.viewModelHandler);
			fixtureDialog.getIcons().add(new Image("images/Kiwi_Logo.png"));
			fixtureDialog.setScene(scene);
			fixtureDialog.setTitle("Lampe ndern/lschen");
			fixtureDialog.showAndWait();
		} else {
			Alert alert = new Alert(AlertType.WARNING);
			Stage stage = (Stage) alert.getDialogPane().getScene().getWindow();
			stage.getIcons().add(new Image("images/Kiwi_Logo.png"));
			alert.setTitle("Warnung");
			alert.setHeaderText("Es existieren keine Lampen.");
			alert.setContentText("Bitte erstellen Sie zuerst eine Lampe.");
			alert.showAndWait();
		}
	}
	// Event Listener on MenuItem.onAction
	@FXML
	public void createGroup(ActionEvent event) throws IOException {
		FXMLLoader loader = new FXMLLoader(getClass().getResource("../group/createPanel/CreateGroup.fxml"));
		Parent createGroupPanel = loader.load();
		CreateGroupController createGroupController = loader.getController();
		Scene scene = new Scene(createGroupPanel);
		Stage groupDialog = new Stage();
		createGroupController.init(groupDialog, viewModelHandler);
		groupDialog.getIcons().add(new Image("images/Kiwi_Logo.png"));
		groupDialog.setScene(scene);
		groupDialog.setTitle("Gruppe hinzufgen");
		groupDialog.showAndWait();
		
		
	}
	// Event Listener on MenuItem.onAction
	@FXML
	public void editDeleteGroup(ActionEvent event) throws IOException {
		FXMLLoader loader = new FXMLLoader(getClass().getResource("../group/editPanel/EditDeleteGroup.fxml"));
		Parent editDeleteGroupPanel = loader.load();
		EditDeleteGroupController editDeleteGroupController = loader.getController();
		Scene scene = new Scene(editDeleteGroupPanel);
		Stage groupDialog = new Stage();
		editDeleteGroupController.init(groupDialog, viewModelHandler);
		groupDialog.getIcons().add(new Image("images/Kiwi_Logo.png"));
		groupDialog.setScene(scene);
		groupDialog.setTitle("Gruppe ndern/lschen");
		groupDialog.showAndWait();
	}
	// Event Listener on MenuItem.onAction
	@FXML
	public void createLightingMood(ActionEvent event) throws IOException {
		FXMLLoader loader = new FXMLLoader(getClass().getResource("../lightingMood/createPanel/CreateLightingMood.fxml"));
		Parent createLightingMoodPanel = loader.load();
		CreateLightingMoodController createLightingMoodController = loader.getController();
		Scene scene = new Scene(createLightingMoodPanel);
		Stage lightingMoodDialog = new Stage();
		createLightingMoodController.init(lightingMoodDialog, this.viewModelHandler.getGroupRepositoryVM());
		lightingMoodDialog.getIcons().add(new Image("images/Kiwi_Logo.png"));
		lightingMoodDialog.setScene(scene);
		lightingMoodDialog.setTitle("Lichtstimmung erstellen");
		lightingMoodDialog.showAndWait();
	}
	// Event Listener on MenuItem.onAction
	@FXML
	public void editDeleteLightingMood(ActionEvent event) throws IOException {
		FXMLLoader loader = new FXMLLoader(getClass().getResource("../lightingMood/editDeletePanel/EditDeleteLightingMood.fxml"));
		Parent editDeleteLightingMoodPanel = loader.load();
		EditDeleteLightingMoodController editDeleteLightingMoodController = loader.getController();
		Scene scene = new Scene(editDeleteLightingMoodPanel);
		Stage lightingMoodDialog = new Stage();
		editDeleteLightingMoodController.init(lightingMoodDialog, this.viewModelHandler);
		lightingMoodDialog.getIcons().add(new Image("images/Kiwi_Logo.png"));
		lightingMoodDialog.setScene(scene);
		lightingMoodDialog.setTitle("Lichtstimmung bearbeiten/lschen");
		lightingMoodDialog.showAndWait();
	}
	// Event Listener on MenuItem.onAction
	@FXML
	public void createComposition(ActionEvent event) throws IOException {
		FXMLLoader loader = new FXMLLoader(getClass().getResource("../composition/createPanel/CreateComposition.fxml"));
		Parent createCompositionPanel = loader.load();
		CreateCompositionController createCompositionController = loader.getController();
		Scene scene = new Scene(createCompositionPanel);
		Stage compositionDialog = new Stage();
		createCompositionController.init(compositionDialog, this.viewModelHandler);
		compositionDialog.getIcons().add(new Image("images/Kiwi_Logo.png"));
		compositionDialog.setScene(scene);
		compositionDialog.setTitle("Komposition erstellen");
		compositionDialog.showAndWait();
		createCompositionController = null;
		loader = null;
		
	}
	// Event Listener on MenuItem.onAction
	@FXML
	public void editDeleteComposition(ActionEvent event) throws IOException {
		FXMLLoader loader = new FXMLLoader(getClass().getResource("../composition/editDeletePanel/EditDeleteComposition.fxml"));
		Parent editDeleteCompositionPanel = loader.load();
		EditDeleteCompositionController editDeleteCompositionController = loader.getController();
		Scene scene = new Scene(editDeleteCompositionPanel);
		Stage compositionDialog = new Stage();
		editDeleteCompositionController.init(compositionDialog, this.viewModelHandler);
		compositionDialog.getIcons().add(new Image("images/Kiwi_Logo.png"));
		compositionDialog.setScene(scene);
		compositionDialog.setTitle("Komposition bearbeiten/lschen");
		compositionDialog.showAndWait();
		editDeleteCompositionController = null;
		loader = null;
	}
	// Event Listener on MenuItem.onAction
	@FXML
	public void createNode(ActionEvent event) {
		Alert alert = new Alert(AlertType.WARNING);
		Stage stage = (Stage) alert.getDialogPane().getScene().getWindow();
		stage.getIcons().add(new Image("images/Kiwi_Logo.png"));
		alert.setTitle("Warnung");
		alert.setHeaderText("Funktion nicht verfgbar.");
		alert.setContentText("Die Funktion zum Speichern von Files ist aktuell noch nicht verfgbar.");
		alert.showAndWait();
	}
	// Event Listener on MenuItem.onAction
	@FXML
	public void editDeleteNode(ActionEvent event) {
		Alert alert = new Alert(AlertType.WARNING);
		Stage stage = (Stage) alert.getDialogPane().getScene().getWindow();
		stage.getIcons().add(new Image("images/Kiwi_Logo.png"));
		alert.setTitle("Warnung");
		alert.setHeaderText("Funktion nicht verfgbar.");
		alert.setContentText("Die Funktion zum Speichern von Files ist aktuell noch nicht verfgbar.");
		alert.showAndWait();
	}
	// Event Listener on MenuItem.onAction
	@FXML
	public void testFixtures(ActionEvent event) {
		Alert alert = new Alert(AlertType.WARNING);
		Stage stage = (Stage) alert.getDialogPane().getScene().getWindow();
		stage.getIcons().add(new Image("images/Kiwi_Logo.png"));
		alert.setTitle("Warnung");
		alert.setHeaderText("Funktion nicht verfgbar.");
		alert.setContentText("Die Funktion zum Speichern von Files ist aktuell noch nicht verfgbar.");
		alert.showAndWait();
	}
	// Event Listener on MenuItem.onAction
	@FXML
	public void openOnlineHelp(ActionEvent event) {
		Desktop desktop = Desktop.isDesktopSupported() ? Desktop.getDesktop() : null;
	    if (desktop != null && desktop.isSupported(Desktop.Action.BROWSE)) {
	        try {
	            desktop.browse(new URI("https://gitlab.cvh-server.de/sboettger/kiwi-home-light-control-system"));
	        } catch (Exception e) {
	            e.printStackTrace();
	        }
	    }
	}
	// Event Listener on MenuItem.onAction
	@FXML
	public void openAboutKIWI(ActionEvent event) {
		String headerText = "KIWI  2024 Sebastian Bttger\n";
		headerText += "Version: 2.0\n\n";
		headerText += "KIWI steht fr KIWI (is a) Illumination Weeny Interface\n";
		headerText += "KIWI verwendet Art-Net 4.\n";
		headerText += "Art-Net Designed by and Copyright Artistic Licence.";
		
		Alert alert = new Alert(AlertType.INFORMATION);
		Stage stage = (Stage) alert.getDialogPane().getScene().getWindow();
		stage.getIcons().add(new Image("images/Kiwi_Logo.png"));
		alert.setTitle("ber KIWI");
		alert.setHeaderText(headerText);
		alert.showAndWait();
	}
}
