package viewModel.composition;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import model.lightingScene.composition.Composition;
import model.lightingScene.composition.CompositionRepository;

public class CompositionRepositoryVM {
	
	private CompositionRepository compositionRepository;
	private ObservableList<CompositionVM> compositionList;
	private PropertyChangeListener compositionListener;
	private CompositionRepositoryVM self = this;
	
	public CompositionRepositoryVM(CompositionRepository compositionRepository) {
		this.compositionRepository = compositionRepository;
		this.compositionList = FXCollections.observableArrayList();
		this.loadCompositions();
		
		this.compositionListener = new PropertyChangeListener() {
			@Override
            public void propertyChange(PropertyChangeEvent event) {
				if(event.getPropertyName().equals("compositions")) {
	    			if(event.getNewValue() != null) {
	    				CompositionVM compositionVM = new CompositionVM((Composition) event.getNewValue());
	    				compositionVM.setCompositionRepositoryVM(self);
	    				compositionList.add(compositionVM);
	    			} else if(event.getOldValue() != null) {
	    				Composition composition = (Composition) event.getOldValue();
	    				int compositionListIndex = -1;
	    				for(int i = 0; i < compositionList.size(); i++) {
	    					if(compositionList.get(i).getCompositionName().equals(composition.getName())) {
	    						compositionListIndex = i;
	    						break;
	    					}
	    				}
	    				if(compositionListIndex != -1) {
	    					compositionList.remove(compositionListIndex);
	    				}
	    			}
	    		}
            }
		};
		this.compositionRepository.addPropertyChangeListener(compositionListener);
	}
	
	public void loadCompositions() {
		this.compositionRepository.getCompositionMap()
			.forEach((String name, Composition composition) -> {
				CompositionVM compositionVM =  new CompositionVM(composition);
				compositionVM.setCompositionRepositoryVM(this);
				this.compositionList.add(compositionVM);
		});
	}
	
	public CompositionVM getComposition(String name) {
		return this.compositionList
				.stream()
				.filter(
					compositionVM -> compositionVM.getCompositionName().equals(name))
				.findFirst()
				.orElse(null);
	}
	
	public ObservableList<CompositionVM> compositionListProperty() {
		return this.compositionList;
	}
	
	public void disselectedOtherCompositons(String compositionName) {
		this.compositionList.forEach(compositionVM -> {
			if(!compositionVM.getCompositionName().equals(compositionName)) {
				compositionVM.setCompositionIsSeleceted(false);
			}
		});
	}

}
