package viewModel.fixture;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javafx.beans.property.BooleanProperty;
import javafx.beans.property.IntegerProperty;
import javafx.beans.property.SimpleBooleanProperty;
import javafx.beans.property.SimpleIntegerProperty;
import javafx.beans.property.SimpleStringProperty;
import javafx.beans.property.StringProperty;
import model.group.Group;

public class FixtureGroupVM {
	
	private IntegerProperty groupId;
	private StringProperty groupName;
	private BooleanProperty fixtureInGroup = new SimpleBooleanProperty(false);
	private BooleanProperty groupSelected = new SimpleBooleanProperty(false);
	private Group group;
	private FixtureGroupRepositoryVM fixtureGroupRepositoryVM;
	private FixtureRepositoryVM fixtureRepositoryVM;
	private PropertyChangeListener groupListener;
	
	public FixtureGroupVM(Group group, FixtureGroupRepositoryVM fixtureGroupRepositoryVM) {
		this.group = group;
		this.fixtureRepositoryVM = new FixtureRepositoryVM(this.group.getFixtureRepository());
		this.fixtureGroupRepositoryVM = fixtureGroupRepositoryVM;
		this.groupId = new SimpleIntegerProperty(group.getId());
		this.groupName = new SimpleStringProperty(group.getName());
		this.groupListener = new PropertyChangeListener() {
			@Override
            public void propertyChange(PropertyChangeEvent event) {
            	if(event.getPropertyName().equals("name")) {
            		groupName.setValue((String)event.getNewValue());
            	}
            }
		};
		
		group.addPropertyChangeListener(this.groupListener);
	}
	
	public String getGroupName() {
		return this.groupName.get();
	}

	public void setGroupName(String groupName) {
		this.group.setName(groupName);
	}
	
	public boolean isFixtureInGroup() {
		return this.fixtureInGroup.get();
	}
	
	public void setFixtureInGroup(boolean inGroup) {
		this.fixtureInGroup.set(inGroup);
	}
	
	public boolean getGroupSelected() {
		return this.groupSelected.get();
	}

	public void setGroupSelected(boolean groupSelected) {
		this.fixtureGroupRepositoryVM.unselectOtherGroups(this.getGroupName());
		this.groupSelected.set(groupSelected);
	}

	public IntegerProperty groupIdProperty() {
		return this.groupId;
	}
	
	public StringProperty groupNameProperty() {
		return this.groupName;
	}
	
	public BooleanProperty fixtureInGroupProperty() {
		return this.fixtureInGroup;
	}
	
	public BooleanProperty groupSelectedProperty() {
		return this.groupSelected;
	}
	
	public FixtureGroupRepositoryVM getFixtureGroupRepositoryVM() {
		return this.fixtureGroupRepositoryVM;
	}
	
	public void addFixtureToGroup(FixtureVM fixtureVM) {
		this.group.addFixture(fixtureVM.getFixture());
	}
	
	public void removeFixtureFromGroup(FixtureVM fixtureVM) {
		this.group.removeFixture(fixtureVM.getFixture());
	}
	
	public FixtureRepositoryVM getFixtureRepositoryVM() {
		return this.fixtureRepositoryVM;
	}
	
	@Override
	protected void finalize() {
	    this.group.removePropertyChangeListener(this.groupListener);
	}
}
