package viewModel.fixture;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import model.fixture.Fixture;
import model.fixture.FixtureRepository;

public class FixtureRepositoryVM {
	private ObservableList<FixtureVM> fixtureList;
    private FixtureRepository fixtureRepository;
    private PropertyChangeListener fixtureRepositoryListener;
    
    public FixtureRepositoryVM(FixtureRepository fixtureRepository) {
    	this.fixtureRepository = fixtureRepository;
    	this.fixtureList = FXCollections.observableArrayList();
    	this.loadFixtures();
    	
    	this.fixtureRepositoryListener = new PropertyChangeListener() {
    		@Override
    		public void propertyChange(PropertyChangeEvent event) {
    			if(event.getPropertyName().equals("fixtures")) {
        			if(event.getNewValue() != null) {
        				Fixture newFixture = (Fixture) event.getNewValue();
        				addFixtureToList(newFixture);
        			} else if(event.getOldValue() != null) {
        				Fixture oldFixture = (Fixture) event.getOldValue();
        				int fixtureListIndex = -1;
        				for(int i = 0; i < fixtureList.size(); i++) {
        					if(fixtureList.get(i).getFixtureName().equals(oldFixture.getName())) {
        						fixtureListIndex = i;
        						break;
        					}
        				}
        				if(fixtureListIndex != -1) {
        					fixtureList.remove(fixtureListIndex);
        				}
        			}
        		}
    		}
    	};
    	
    	this.fixtureRepository.addPropertyChangeListener(this.fixtureRepositoryListener);
    }
    
    private void loadFixtures() {
    	this.fixtureRepository.getFixtureMap().forEach((String name, Fixture fixture) -> {
    		this.addFixtureToList(fixture);
    	});
    }
    
    private void addFixtureToList(Fixture fixture) {
    	FixtureVM newFixture = new FixtureVM(fixture, this);
    	this.fixtureList.add(newFixture);
    }
    
    public ObservableList<FixtureVM> fixturesRepositoryProperty() {
    	return this.fixtureList;
    }
    
    public void newFixture(String name, int address) {
    	this.fixtureRepository.addFixture(new Fixture(name, address));
    }
    
    public void deleteFixture(String name) {
    	this.fixtureRepository.removeFixture(name);
    }
    
    public FixtureVM getFixtureVM(String name) {
    	return this.fixtureList.stream().filter(fixtureVM -> fixtureVM.getFixtureName().equals(name)).findFirst().orElse(null);
    }
    
    public ObservableList<FixtureVM> fixtureListProperty() {
    	return this.fixtureList;
    }
    
    public void unselectOtherFixtures(String fixtureName) {
    	this.fixtureList.forEach(fixtureVM -> {
    		if(!fixtureVM.getFixtureName().equals(fixtureName) && fixtureVM.getFixtureSelected()) {
    			fixtureVM.setFixtureSelected(false);
    		}
    	});
    }
    
    @Override
	protected void finalize() {
	    this.fixtureRepository.removePropertyChangeListener(this.fixtureRepositoryListener);
	}
}
