package viewModel.fixture;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javafx.beans.property.BooleanProperty;
import javafx.beans.property.IntegerProperty;
import javafx.beans.property.SimpleBooleanProperty;
import javafx.beans.property.SimpleIntegerProperty;
import javafx.beans.property.SimpleStringProperty;
import javafx.beans.property.StringProperty;
import model.fixture.Fixture;

public class FixtureVM {
	
	private StringProperty fixtureName;
	private IntegerProperty fixtureAddress;
	private Fixture fixture;
	private BooleanProperty fixtureInGroup = new SimpleBooleanProperty(false);
	private BooleanProperty fixtureSelected = new SimpleBooleanProperty(false);
	private PropertyChangeListener fixtureListener;
	private FixtureRepositoryVM fixtureRepositoryVM;
	private DmxValuesVM dmxValuesVM;
	private String lightingMoodName = "Benutzerdefeniert";
		
	public FixtureVM(Fixture fixture, FixtureRepositoryVM fixtureRepositoryVM) {
		this.fixture = fixture;
		this.fixtureRepositoryVM = fixtureRepositoryVM;
		this.dmxValuesVM = new DmxValuesVM(fixture.getDmxValues());
		this.fixtureName = new SimpleStringProperty(this.fixture.getName());
		this.fixtureAddress = new SimpleIntegerProperty(this.fixture.getAddress());
		this.fixtureListener = new PropertyChangeListener() {
			@Override
            public void propertyChange(PropertyChangeEvent event) {
				if(event.getPropertyName().equals("name")) {
					fixtureName.setValue((String)event.getNewValue());
				} else if(event.getPropertyName().equals("address")) {
					fixtureAddress.setValue((Integer)event.getNewValue());
				}
			}
		};
		this.fixture.addPropertyChangeListener(this.fixtureListener);
	}

	public String getFixtureName() {
		return fixtureName.get();
	}

	public void setFixtureName(String fixtureName) {
		this.fixture.setName(fixtureName);
	}

	public int getFixtureAddress() {
		return fixtureAddress.get();
	}

	public void setFixtureAddress(int fixtureAddress) {
		this.fixture.setAddress(fixtureAddress);
	}
	
	public boolean getFixtureSelected() {
		return this.fixtureSelected.get();
	}
	
	public void setFixtureSelected(boolean selected) {
		this.fixtureRepositoryVM.unselectOtherFixtures(this.fixtureName.get());
		this.fixtureSelected.set(selected);
	}
	
	public void setFixtureInGroup(boolean inGroup) {
		this.fixtureInGroup.set(inGroup);
	}
	
	public boolean isFixtureInGroup() {
		return this.fixtureInGroup.get();
	}
	
	public StringProperty fixtureNameProperty() {
		return this.fixtureName;
	}
	
	public IntegerProperty fixtureAddressProperty() {
		return this.fixtureAddress;
	}
	
	public Fixture getFixture() {
		return this.fixture;
	}
	
	public BooleanProperty fixtureInGroupProperty() {
		return this.fixtureInGroup;
	}
	
	public BooleanProperty fixtureSelectedProperty() {
		return this.fixtureSelected;
	}
	
	public DmxValuesVM getDmxValuesVM() {
		return this.dmxValuesVM;
	}
	
	public String getLightingMoodName() {
		return lightingMoodName;
	}

	public void setLightingMoodName(String lightingMoodName) {
		this.lightingMoodName = lightingMoodName;
	}

	@Override
	protected void finalize() {
	    this.fixture.removePropertyChangeListener(this.fixtureListener);
	}
}
