package viewModel.group;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javafx.beans.property.ObjectProperty;
import javafx.beans.property.SimpleObjectProperty;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import model.KiwiModelHandler;
import model.group.Group;
import model.group.GroupReposirory;

public class GroupRepositoryVM {
    private ObservableList<GroupVM> groupsList;
    private GroupReposirory groupRepository;
    private PropertyChangeListener groupRepositoryListener;
    private ObjectProperty<GroupVM> currentSelectedGroup = new SimpleObjectProperty<>();
    

    public GroupRepositoryVM(GroupReposirory groupRepository) {
    	this.groupRepository = groupRepository;
    	this.groupsList = FXCollections.observableArrayList();
    	this.loadGroups();
    	this.groupRepositoryListener = new PropertyChangeListener() {
			@Override
            public void propertyChange(PropertyChangeEvent event) {
				if(event.getPropertyName().equals("groups")) {
	    			if(event.getNewValue() != null) {
	    				Group newGroup = (Group) event.getNewValue();
	    				addGroupToList(newGroup);
	    			} else if(event.getOldValue() != null) {
	    				Group oldGroup = (Group) event.getOldValue();
	    				int groupListIndex = -1;
	    				for(int i = 0; i < groupsList.size(); i++) {
	    					if(groupsList.get(i).getGroupName().equals(oldGroup.getName())) {
	    						groupListIndex = i;
	    						break;
	    					}
	    				}
	    				if(groupListIndex != -1) {
	    					groupsList.remove(groupListIndex);
	    				}
	    			}
	    		}
            }
    	};
    	this.groupRepository.addPropertyChangeListener(this.groupRepositoryListener);
    }

    private void loadGroups() {
    	this.groupRepository.getGroupsMap().forEach((String name, Group group) -> {
    		this.addGroupToList(group);
    	});
    }
    
    private void addGroupToList(Group group) {
    	this.groupsList.add(new GroupVM(group, this));
    }
    
    public ObservableList<GroupVM> groupsListProperty() {
    	return this.groupsList;
    }
    
    public void newGroup(String name) {
    	this.groupRepository.addGroup(new Group(name, KiwiModelHandler.getLightController()));
    }
    
    public void deleteGroup(String name) {
    	this.groupRepository.removeGroup(name);
    }

    public GroupVM getGroupVM(String name) {
    	return this.groupsList.stream()
    			.filter(groupVM -> groupVM
    					.getGroupName().equals(name))
    			.findFirst()
    			.orElse(null);
    }
    
    public void unselectOtherGroups(String groupName) {
    	this.currentSelectedGroup.set(this.groupsList
    			.stream()
    			.filter(groupVM -> groupVM.getGroupName() == groupName)
    			.findFirst()
    			.orElse(null));
    	
    	this.groupsList.forEach(groupVM -> {
    		if(!groupVM.getGroupName().equals(groupName) && groupVM.getGroupSelected()) {
    			groupVM.setGroupSelected(false);
    		}
    	});
    }

    public ObjectProperty<GroupVM> currentSelectedGroupProperty() {
    	return this.currentSelectedGroup;
    }
    
    @Override
   	protected void finalize() {
   	    this.groupRepository.removePropertyChangeListener(this.groupRepositoryListener);
   	}
}
