package viewModel.group;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import javafx.beans.property.BooleanProperty;
import javafx.beans.property.IntegerProperty;
import javafx.beans.property.ObjectProperty;
import javafx.beans.property.SimpleBooleanProperty;
import javafx.beans.property.SimpleIntegerProperty;
import javafx.beans.property.SimpleObjectProperty;
import javafx.beans.property.SimpleStringProperty;
import javafx.beans.property.StringProperty;
import model.group.Group;
import viewModel.composition.CompositionRepositoryVM;
import viewModel.composition.CompositionVM;
import viewModel.fixture.FixtureRepositoryVM;
import viewModel.fixture.FixtureVM;
import viewModel.lightingMood.LightingMoodRepositoryVM;
import viewModel.lightingMood.LightingMoodVM;

public class GroupVM {
	
	public static enum State{
		OFF,
		MID,
		ON
	};
	
	private IntegerProperty groupId;
	private StringProperty groupName;
	private ObjectProperty<State> groupState = new SimpleObjectProperty<>(State.OFF);
	private BooleanProperty groupSelected = new SimpleBooleanProperty(false);
	private Group group;
	private GroupRepositoryVM groupRepositoryVM;
	private PropertyChangeListener groupListener;
	private LightingMoodRepositoryVM lightingMoodRepositoryVM;
	private CompositionRepositoryVM compositionRepositoryVM;
	private FixtureRepositoryVM fixtureRepositoryVM;
	
	public GroupVM(Group group, GroupRepositoryVM groupRepositoryVM) {
		this.group = group;
		this.groupRepositoryVM = groupRepositoryVM;
		this.fixtureRepositoryVM = new FixtureRepositoryVM(this.group.getFixtureRepository());
		this.lightingMoodRepositoryVM = new LightingMoodRepositoryVM(this.group.getLightingMoodRepository());
		this.compositionRepositoryVM = new CompositionRepositoryVM(this.group.getCompositionRepository());
		this.groupId = new SimpleIntegerProperty(group.getId());
		this.groupName = new SimpleStringProperty(group.getName());
		this.groupListener = new PropertyChangeListener() {
			@Override
            public void propertyChange(PropertyChangeEvent event) {
				if(event.getPropertyName().equals("name")) {
					groupName.setValue((String)event.getNewValue());
				} else if(event.getPropertyName().equals("currentState")) {
					switch((model.group.Group.State) event.getNewValue()) {
						case AKTIV:
							groupState.setValue(State.ON);
							break;
						case PARTAKTIV:
							groupState.setValue(State.MID);
							break;
						case DEAKTIV:
							groupState.setValue(State.OFF);
							break;
					}
				}
            }
		};
		
		group.addPropertyChangeListener(this.groupListener);
		this.groupState.addListener((object, oldValue, newValue) -> {
			this.setGroupState((State) newValue);
		});
	}
	
	public String getGroupName() {
		return this.groupName.get();
	}

	public void setGroupName(String groupName) {
		this.group.setName(groupName);
	}

	public State getGroupState() {
		return this.groupState.get();
	}

	public void setGroupState(State groupState) {
		switch(groupState) {
			case ON:
				this.group.activate();
				break;
			case MID:
				this.group.setCurrentState(model.group.Group.State.PARTAKTIV);
				break;
			case OFF:
				this.group.deactivate();
				break;
		}
	}

	public boolean getGroupSelected() {
		return this.groupSelected.get();
	}

	public void setGroupSelected(boolean groupSelected) {
		if(groupSelected) {
			this.groupRepositoryVM.unselectOtherGroups(this.getGroupName());
		}
		this.groupSelected.set(groupSelected);
	}

	public FixtureRepositoryVM getFixtureRepositoryVM() {
		return this.fixtureRepositoryVM;
	}
	
	public LightingMoodRepositoryVM getLightingMoodRepositoryVM() {
		return this.lightingMoodRepositoryVM;
	}
	
	public CompositionRepositoryVM getCompositionRepositoryVM() {
		return this.compositionRepositoryVM;
	}

	public IntegerProperty groupIdProperty() {
		return this.groupId;
	}
	
	public StringProperty groupNameProperty() {
		return this.groupName;
	}

	public ObjectProperty<State> groupStateProperty() {
		return this.groupState;
	}

	public BooleanProperty groupSelectedProperty() {
		return this.groupSelected;
	}
	
	public Group getGroup() {
		return this.group;
	}
	
	public void addFixture(FixtureVM fixtureVM) {
		this.group.addFixture(fixtureVM.getFixture());
	}
	
	public void removeFixture(FixtureVM fixtureVM) {
		this.group.removeFixture(fixtureVM.getFixture());
	}
	
	public boolean isFixtureInGroup(String name) {
		if(this.group.getFixtureRepository().getFixture(name) == null) {
			return false;
		} else {
			return true;
		}
	}
	
	public void addLightingMood(LightingMoodVM lightingMoodVM) {
		this.group.addLightingMood(lightingMoodVM.getLightingMood());
	}
	
	public void activateLightingMood(LightingMoodVM lightingMoodVM) {
		this.group.activateLightingMood(
				this.lightingMoodRepositoryVM
					.getLightingMood(lightingMoodVM.getLightingMoodName()).getLightingMood());
	}
	
	public void deactivateLightingMood(LightingMoodVM lightingMoodVM) {
		this.group.deactivateLightingMood(
				this.lightingMoodRepositoryVM
					.getLightingMood(lightingMoodVM.getLightingMoodName()).getLightingMood());
	}
	
	public void removeLightingMood(LightingMoodVM ligtingMoodVM) {
		this.group.removeLightingMood(ligtingMoodVM.getLightingMood());
	}
	
	public void addComposition(CompositionVM compositionVM) {
		this.group.addComposition(compositionVM.getComposition());
	}
	
	public void activateComposition(CompositionVM compositionVM) {
		this.group.activateComposition(compositionVM.getComposition());
	}
	
	public void deactivateComposition(CompositionVM compositionVM) {
		this.group.deactivateComposition(compositionVM.getComposition());
	}

	public void removeComposition(CompositionVM composition) {
		this.group.removeComposition(composition.getComposition());
	}
	
	@Override
 	protected void finalize() {
	    this.group.removePropertyChangeListener(this.groupListener);
	}
}
