#ifndef ArtNetController_h
#define ArtNetController_h

#include <SPI.h>
#include <Arduino.h>
#include <Ethernet.h>
#include <EthernetUdp.h>
#include <ArduinoJson.h>
#include "./ArtCommands.h"
#include "../config/config.h"
#include "./ArtNetPackageTypes.h"
#include "../DmxController/DmxController.h"
#include "../PresetRepository/PresetRepository.h"

class ArtNetController
{
    private:
        PresetRepository* presetRepository;
        DmxController* dmxController;
        uint8_t mac[6];
        uint8_t nodeIpAddress[4];
        uint8_t controllerIpAddress[4] = {0x00, 0x00, 0x00, 0x00};
        EthernetUDP udp;
        int startUdp();
        bool checkArtNetHeader(uint8_t udpData[]);
        uint8_t *getPresetStatus();
        StaticJsonDocument<50> doc;

    public:
        ArtNetController();
        ArtNetController(DmxController* dmxController, PresetRepository* presetRepository);
        void sendPresetActivated(int presetId);
        void sendPresetDeactivated(int presetId);
        void listenToArtNet();
        ArtPoll parseToArtPoll(uint8_t updData[], int length);
        ArtCommand parseToArtCommand(uint8_t updData[], int length);
        ArtDataRequest parseToArtDataRequest(uint8_t updData[], int length);
        ArtDmx parseToArtDmx(uint8_t updData[], int length);
        void parseFromArtPollReply(ArtPollReply artPollReply, uint8_t* udpSendData, int length);
        void parseFromArtCommand(ArtCommand artCommand, uint8_t* udpSendData, int length);
        void parseFromArtDataReply(ArtDataReply artDataReply, uint8_t* udpSendData, int length);
        void sendArtPollReply();
        void sendArtCommand(char *message, uint16_t length);
        void sendArtDataReply(uint16_t dataRequstCode);
        void processArtCommand(uint8_t updData[], int length);
        void processArtDataRequest(uint8_t updData[], int length);
        void processArtDmx(uint8_t updData[], int length);
};
#endif